import logging

from angr.procedures.stubs.format_parser import FormatParser

l = logging.getLogger(name=__name__)

######################################
# snprintf
######################################


class snprintf(FormatParser):
    def run(self, dst_ptr, size, fmt):  # pylint:disable=arguments-differ,unused-argument
        if self.state.solver.eval(size) == 0:
            return size

        fmt_str = self._parse(fmt)
        out_str = fmt_str.replace(self.va_arg)
        self.state.memory.store(dst_ptr, out_str)

        # place the terminating null byte
        self.state.memory.store(dst_ptr + (out_str.size() // self.arch.byte_width), self.state.solver.BVV(0, 8))

        return out_str.size() // self.arch.byte_width


######################################
# __snprintf_chk
######################################


class __snprintf_chk(FormatParser):
    def run(self, dst_ptr, maxlen, flag, size, fmt):  # pylint:disable=arguments-differ,unused-argument
        # The format str is at index 4
        fmt_str = self._parse(fmt)
        out_str = fmt_str.replace(self.va_arg)
        self.state.memory.store(dst_ptr, out_str)

        # place the terminating null byte
        self.state.memory.store(dst_ptr + (out_str.size() // self.arch.byte_width), self.state.solver.BVV(0, 8))

        return out_str.size() // self.arch.byte_width



# pylint: disable=missing-class-docstring,disable=no-self-use
import os
import unittest

import angr

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")


class TestIatResolver(unittest.TestCase):
    def test_iat(self):
        p = angr.Project(os.path.join(test_location, "i386", "simple_windows.exe"), auto_load_libs=False)
        cfg = p.analyses.CFGFast()

        strcmp_caller_bb = cfg.get_any_node(0x401010)
        assert len(strcmp_caller_bb.successors) == 1

        strcmp = strcmp_caller_bb.successors[0]
        assert strcmp.is_simprocedure
        assert strcmp.simprocedure_name == "strcmp"

        strcmp_successors = strcmp.successors
        assert len(strcmp_successors) == 1

        strcmp_ret_to = strcmp_successors[0]
        assert strcmp_ret_to.addr == 0x40102A


if __name__ == "__main__":
    unittest.main()



import os


import angr
import angr.calling_conventions

from angr.sim_type import SimTypePointer

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")


def test_function_prototype():
    proj = angr.Project(os.path.join(test_location, "x86_64", "all"), auto_load_libs=False)

    func = angr.knowledge_plugins.Function(proj.kb.functions, 0x100000, name="strcmp")
    func.prototype = angr.SIM_LIBRARIES["libc.so.6"].prototypes[func.name]
    func.calling_convention = angr.calling_conventions.DEFAULT_CC[proj.arch.name](proj.arch)


def test_find_prototype():
    proj = angr.Project(os.path.join(test_location, "x86_64", "all"), auto_load_libs=False)

    cfg = proj.analyses.CFG()

    func = cfg.kb.functions.function(name="strcmp", plt=False)
    func.calling_convention = angr.calling_conventions.DEFAULT_CC[proj.arch.name](proj.arch)

    func.find_declaration()

    arg_locs = func.calling_convention.arg_locs(func.prototype)

    assert len(arg_locs) == 2
    assert arg_locs[0].reg_name == "rdi"
    assert arg_locs[1].reg_name == "rsi"


def test_cpp_void_pointer():
    proj = angr.Project(os.path.join(test_location, "x86_64", "void_pointer"), auto_load_libs=False)

    cfg = proj.analyses.CFG()
    proj.analyses.CompleteCallingConventions(recover_variables=True, analyze_callsites=True)

    func = cfg.kb.functions.function(name="_ZdlPvm", plt=True)
    assert isinstance(func.prototype.args[0], SimTypePointer)


def main():
    test_find_prototype()
    test_function_prototype()


if __name__ == "__main__":
    main()



import gc
import os
import pickle
import re

import angr
from angr import options as so

from common import slow_test

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..")


def _remove_addr_from_trace_item(trace_item_str):
    m = re.match(r"(<\S+ \S+) from 0x[0-9a-f]+(:[\s\S]+)", trace_item_str)
    if m is None:
        return None
    return m.group(1) + m.group(2)


def _compare_trace(trace, expected):
    assert len(trace) == len(expected)

    for trace_item, expected_str in zip(trace, expected):
        trace_item_str = str(trace_item)
        if trace_item_str.startswith("<SimProcedure"):
            # we do not care if addresses of SimProcedures match, since they are not allocated in a deterministic way
            trace_item_str = _remove_addr_from_trace_item(trace_item_str)
            expected_str = _remove_addr_from_trace_item(expected_str)

        assert trace_item_str == expected_str


def test_stops():
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "i386", "uc_stop"), auto_load_libs=False)

    # test STOP_NORMAL, STOP_STOPPOINT
    s_normal = p.factory.entry_state(args=["a"], add_options=so.unicorn)
    s_normal.unicorn.max_steps = 100
    pg_normal = p.factory.simulation_manager(s_normal).run()
    p_normal = pg_normal.one_deadended
    _compare_trace(
        p_normal.history.descriptions,
        [
            "<Unicorn (STOP_STOPPOINT after 4 steps) from 0x8048340: 1 sat>",
            "<SimProcedure __libc_start_main from 0x8119990: 1 sat>",
            "<Unicorn (STOP_STOPPOINT after 14 steps) from 0x8048650: 1 sat>",
            "<SimProcedure __libc_start_main from 0x8400044: 1 sat>",
            "<Unicorn (STOP_NORMAL after 100 steps) from 0x80485b5: 1 sat>",
            "<Unicorn (STOP_STOPPOINT after 12 steps) from 0x804846f: 1 sat>",
            "<SimProcedure __libc_start_main from 0x8400048: 1 sat>",
        ],
    )

    s_normal_angr = p.factory.entry_state(args=["a"])
    pg_normal_angr = p.factory.simulation_manager(s_normal_angr).run()
    p_normal_angr = pg_normal_angr.one_deadended
    assert p_normal_angr.history.bbl_addrs.hardcopy == p_normal.history.bbl_addrs.hardcopy

    # test STOP_STOPPOINT on an address that is not a basic block start
    s_stoppoints = p.factory.call_state(
        p.loader.find_symbol("main").rebased_addr, 1, angr.PointerWrapper([]), add_options=so.unicorn
    )

    # this address is right before/after the bb for the stop_normal() function ends
    # we should not stop there, since that code is never hit
    stop_fake = [0x0804847C, 0x08048454]

    # this is an address inside main that is not the beginning of a basic block. we should stop here
    stop_in_bb = 0x08048638
    stop_bb = 0x08048633  # basic block of the above address
    pg_stoppoints = p.factory.simulation_manager(s_stoppoints).run(n=1, extra_stop_points=stop_fake + [stop_in_bb])
    assert len(pg_stoppoints.active) == 1
    p_stoppoints = pg_stoppoints.one_active
    assert p_stoppoints.addr == stop_bb
    _compare_trace(
        p_stoppoints.history.descriptions,
        ["<Unicorn (STOP_STOPPOINT after 111 steps) from 0x80485b5: 1 sat>"],
    )

    # test STOP_SYMBOLIC_READ_SYMBOLIC_TRACKING_DISABLED
    s_symbolic_read_tracking_disabled = p.factory.entry_state(
        args=["a", "a"],
        add_options=so.unicorn,
        remove_options={so.UNICORN_SYM_REGS_SUPPORT},
    )
    pg_symbolic_read_tracking_disabled = p.factory.simulation_manager(s_symbolic_read_tracking_disabled).run()
    p_symbolic_read_tracking_disabled = pg_symbolic_read_tracking_disabled.one_deadended
    _compare_trace(
        p_symbolic_read_tracking_disabled.history.descriptions,
        [
            "<Unicorn (STOP_STOPPOINT after 4 steps) from 0x8048340: 1 sat>",
            "<SimProcedure __libc_start_main from 0x8119990: 1 sat>",
            "<Unicorn (STOP_STOPPOINT after 14 steps) from 0x8048650: 1 sat>",
            "<SimProcedure __libc_start_main from 0x8400044: 1 sat>",
            "<Unicorn (STOP_SYMBOLIC_READ_SYMBOLIC_TRACKING_DISABLED after 7 steps) from 0x80485b5: 1 sat>",
            "<IRSB from 0x804848a: 1 sat 3 unsat>",
            "<Unicorn (STOP_STOPPOINT after 3 steps) from 0x80484bb: 1 sat>",
            "<SimProcedure __libc_start_main from 0x8400048: 1 sat>",
        ],
    )

    s_symbolic_read_tracking_disabled_angr = p.factory.entry_state(args=["a", "a"])
    pg_symbolic_read_tracking_disabled_angr = p.factory.simulation_manager(s_symbolic_read_tracking_disabled_angr).run()
    p_symbolic_read_tracking_disabled_angr = pg_symbolic_read_tracking_disabled_angr.one_deadended
    assert (
        p_symbolic_read_tracking_disabled_angr.history.bbl_addrs.hardcopy
        == p_symbolic_read_tracking_disabled.history.bbl_addrs.hardcopy
    )

    # test STOP_SEGFAULT
    s_segfault = p.factory.entry_state(
        args=["a", "a", "a", "a", "a", "a", "a"],
        add_options=so.unicorn | {so.STRICT_PAGE_ACCESS, so.ENABLE_NX},
    )
    pg_segfault = p.factory.simulation_manager(s_segfault).run()
    p_segfault = pg_segfault.errored[0].state
    # TODO: fix the permissions segfault to commit if it's a MEM_FETCH
    # this will extend the last simunicorn one more block
    _compare_trace(
        p_segfault.history.descriptions,
        [
            "<Unicorn (STOP_STOPPOINT after 4 steps) from 0x8048340: 1 sat>",
            "<SimProcedure __libc_start_main from 0x8119990: 1 sat>",
            "<Unicorn (STOP_STOPPOINT after 14 steps) from 0x8048650: 1 sat>",
            "<SimProcedure __libc_start_main from 0x8400044: 1 sat>",
            "<Unicorn (STOP_SEGFAULT after 7 steps) from 0x80485b5: 1 sat>",
            "<IRSB from 0x8048508: 1 sat>",
        ],
    )

    s_segfault_angr = p.factory.entry_state(
        args=["a", "a", "a", "a", "a", "a", "a"],
        add_options={so.STRICT_PAGE_ACCESS, so.ENABLE_NX},
    )
    pg_segfault_angr = p.factory.simulation_manager(s_segfault_angr).run()
    p_segfault_angr = pg_segfault_angr.errored[0].state
    assert p_segfault_angr.history.bbl_addrs.hardcopy == p_segfault.history.bbl_addrs.hardcopy
    assert pg_segfault_angr.errored[0].error.addr == pg_segfault.errored[0].error.addr

    # test STOP_SYMBOLIC_BLOCK_EXIT
    s_symbolic_exit = p.factory.entry_state(args=["a"] * 10, add_options=so.unicorn)
    pg_symbolic_exit = p.factory.simulation_manager(s_symbolic_exit).run()
    p_symbolic_exit = pg_symbolic_exit.one_deadended
    _compare_trace(
        p_symbolic_exit.history.descriptions,
        [
            "<Unicorn (STOP_STOPPOINT after 4 steps) from 0x8048340: 1 sat>",
            "<SimProcedure __libc_start_main from 0x8119990: 1 sat>",
            "<Unicorn (STOP_STOPPOINT after 14 steps) from 0x8048650: 1 sat>",
            "<SimProcedure __libc_start_main from 0x8400044: 1 sat>",
            "<Unicorn (STOP_SYMBOLIC_BLOCK_EXIT_CONDITION after 7 steps) from 0x80485b5: 1 sat>",
            "<IRSB from 0x804855d: 2 sat 1 unsat>",
            "<Unicorn (STOP_STOPPOINT after 4 steps) from 0x8048587: 1 sat>",
            "<SimProcedure __libc_start_main from 0x8400048: 1 sat>",
        ],
    )

    s_symbolic_exit_angr = p.factory.entry_state(args=["a"] * 10)
    pg_symbolic_exit_angr = p.factory.simulation_manager(s_symbolic_exit_angr).run()
    p_symbolic_exit_angr = pg_symbolic_exit_angr.one_deadended
    assert p_symbolic_exit_angr.history.bbl_addrs.hardcopy == p_symbolic_exit.history.bbl_addrs.hardcopy


def run_longinit(arch):
    p = angr.Project(os.path.join(test_location, "binaries", "tests", arch, "longinit"), auto_load_libs=False)
    s_unicorn = p.factory.entry_state(add_options=so.unicorn, remove_options={so.SHORT_READS})
    pg = p.factory.simulation_manager(s_unicorn, save_unconstrained=True, save_unsat=True)
    pg.explore()
    s = pg.deadended[0]
    (first, _), (second, _) = s.posix.stdin.content
    s.add_constraints(first == s.solver.BVV(b"A" * 9))
    s.add_constraints(second == s.solver.BVV(b"B" * 9))
    assert s.posix.dumps(1) == b"You entered AAAAAAAAA and BBBBBBBBB!\n"


def test_longinit_i386():
    run_longinit("i386")


def test_longinit_x86_64():
    run_longinit("x86_64")


def broken_fauxware_arm():
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "armel", "fauxware"), auto_load_libs=False)
    s_unicorn = p.factory.entry_state(add_options=so.unicorn)  # unicorn
    pg = p.factory.simulation_manager(s_unicorn)
    pg.explore()
    assert all("Unicorn" in "".join(p.history.descriptions.hardcopy) for p in pg.deadended)
    assert sorted(pg.mp_deadended.posix.dumps(1).mp_items) == sorted(
        (
            b"Username: \nPassword: \nWelcome to the admin console, trusted user!\n",
            b"Username: \nPassword: \nGo away!",
            b"Username: \nPassword: \nWelcome to the admin console, trusted user!\n",
        )
    )


def test_fauxware():
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "i386", "fauxware"), auto_load_libs=False)
    s_unicorn = p.factory.entry_state(add_options=so.unicorn)  # unicorn
    pg = p.factory.simulation_manager(s_unicorn)
    pg.explore()

    assert all("Unicorn" in "".join(p.history.descriptions.hardcopy) for p in pg.deadended)
    assert sorted(pg.mp_deadended.posix.dumps(1).mp_items) == sorted(
        (
            b"Username: \nPassword: \nWelcome to the admin console, trusted user!\n",
            b"Username: \nPassword: \nGo away!",
            b"Username: \nPassword: \nWelcome to the admin console, trusted user!\n",
        )
    )


def test_fauxware_aggressive():
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "i386", "fauxware"), auto_load_libs=False)
    s_unicorn = p.factory.entry_state(
        add_options=so.unicorn | {so.UNICORN_AGGRESSIVE_CONCRETIZATION},
        remove_options={so.LAZY_SOLVES},
    )  # unicorn
    s_unicorn.unicorn.cooldown_symbolic_stop = 2
    s_unicorn.unicorn.cooldown_unsupported_stop = 2
    s_unicorn.unicorn.cooldown_nonunicorn_blocks = 0

    pg = p.factory.simulation_manager(s_unicorn)
    pg.explore()

    assert len(pg.deadended) == 1


def test_partial_reads():
    """
    This test case if unicorn engine correctly handles case when symbolic taint is introduced by the second partial read
    performed by unicorn. Unicorn triggers memory read hook twice when reading value greater than 8 bytes on x86-64.
    """

    p = angr.Project(
        os.path.join(test_location, "binaries", "tests", "x86_64", "test_partial_reads_handling_in_unicorn"),
        auto_load_libs=False,
    )
    # Do not treat as uninitialized memory as symbolic. Prevents introducing undesired symbolic taint
    init_state = p.factory.full_init_state(add_options=so.unicorn | {so.ZERO_FILL_UNCONSTRAINED_MEMORY})
    global_var_val = [
        init_state.solver.BVV(0x41414141, 32),
        init_state.solver.BVV(0x42424242, 32),
        init_state.solver.BVS("symb_val_0", 32),
        init_state.solver.BVS("symb_val_1", 32),
    ]
    global_var_symb = p.loader.find_symbol("global_var")
    # Store every byte separately so that entire variable is not treated as symbolic
    for count, val in enumerate(global_var_val):
        init_state.memory.store(global_var_symb.rebased_addr + count * 4, val, endness=init_state.arch.memory_endness)

    pg = p.factory.simulation_manager(init_state)
    pg.run()
    assert len(pg.deadended) == 1


def run_similarity(binpath, depth, prehook=None):
    b = angr.Project(os.path.join(test_location, binpath), auto_load_libs=False)
    cc = b.analyses.CongruencyCheck(throw=True)
    cc.set_state_options(
        left_add_options=so.unicorn,
        left_remove_options={
            so.LAZY_SOLVES,
            so.TRACK_MEMORY_MAPPING,
            so.COMPOSITE_SOLVER,
        },
        right_add_options={so.ZERO_FILL_UNCONSTRAINED_REGISTERS},
        right_remove_options={
            so.LAZY_SOLVES,
            so.TRACK_MEMORY_MAPPING,
            so.COMPOSITE_SOLVER,
        },
    )
    if prehook:
        cc.simgr = prehook(cc.simgr)
    cc.run(depth=depth)


@slow_test
def test_similarity_fauxware():
    def cooldown(pg):
        # gotta skip the initializers because of cpuid and RDTSC
        pg.one_left.unicorn.countdown_nonunicorn_blocks = 39
        return pg

    run_similarity(os.path.join("binaries", "tests", "i386", "fauxware"), 1000, prehook=cooldown)


def test_fp():
    with open(os.path.join(test_location, "binaries", "tests_src", "manyfloatsum.c")) as fp:
        type_cache = angr.sim_type.parse_defns(fp.read())
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "i386", "manyfloatsum"), auto_load_libs=False)

    for function in (
        "sum_floats",
        "sum_combo",
        "sum_segregated",
        "sum_doubles",
        "sum_combo_doubles",
        "sum_segregated_doubles",
    ):
        args = list(range(len(type_cache[function].args)))
        answer = float(sum(args))
        addr = p.loader.find_symbol(function).rebased_addr
        my_callable = p.factory.callable(addr, prototype=type_cache[function])
        my_callable.set_base_state(p.factory.blank_state(add_options=so.unicorn))
        result = my_callable(*args)
        assert not result.symbolic
        result_concrete = result.args[0]
        assert answer == result_concrete


def test_unicorn_pickle():
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "i386", "fauxware"), auto_load_libs=False)

    def _uni_state():
        # try pickling out paths that went through unicorn
        s_unicorn = p.factory.entry_state(add_options=so.unicorn)
        s_unicorn.unicorn.countdown_nonunicorn_blocks = 0
        s_unicorn.unicorn.countdown_symbolic_stop = 0
        s_unicorn.unicorn.cooldown_nonunicorn_blocks = 0
        s_unicorn.unicorn.cooldown_symbolic_stop = 2
        return s_unicorn

    pg = p.factory.simulation_manager(_uni_state())
    pg.one_active.options.update(so.unicorn)
    pg.run(until=lambda lpg: "Unicorn" in lpg.one_active.history.recent_description)
    assert len(pg.active) > 0

    pgp = pickle.dumps(pg, -1)
    del pg
    gc.collect()
    pg2 = pickle.loads(pgp)
    pg2.explore()

    assert sorted(pg2.mp_deadended.posix.dumps(1).mp_items) == sorted(
        (
            b"Username: \nPassword: \nWelcome to the admin console, trusted user!\n",
            b"Username: \nPassword: \nGo away!",
            b"Username: \nPassword: \nWelcome to the admin console, trusted user!\n",
        )
    )

    # test the pickling of SimUnicorn itself
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "i386", "fauxware"), auto_load_libs=False)
    pg = p.factory.simulation_manager(_uni_state())
    pg.run(n=2)
    assert p.factory.successors(pg.one_active).sort == "Unicorn"

    pgp = pickle.dumps(pg, -1)
    del pg
    gc.collect()
    pg2 = pickle.loads(pgp)
    pg2.explore()

    assert sorted(pg2.mp_deadended.posix.dumps(1).mp_items) == sorted(
        (
            b"Username: \nPassword: \nWelcome to the admin console, trusted user!\n",
            b"Username: \nPassword: \nGo away!",
            b"Username: \nPassword: \nWelcome to the admin console, trusted user!\n",
        )
    )


def test_concrete_transmits():
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "cgc", "PIZZA_00001"), auto_load_libs=False)
    inp = bytes.fromhex("320a310a0100000005000000330a330a340a")

    s_unicorn = p.factory.entry_state(
        add_options=so.unicorn | {so.CGC_NO_SYMBOLIC_RECEIVE_LENGTH},
        stdin=inp,
        flag_page=b"\0" * 4096,
    )
    pg_unicorn = p.factory.simulation_manager(s_unicorn)
    pg_unicorn.run(n=10)

    assert pg_unicorn.one_active.posix.dumps(1) == (
        b"1) Add number to the array\n"
        b"2) Add random number to the array\n"
        b"3) Sum numbers\n"
        b"4) Exit\n"
        b"Randomness added\n"
        b"1) Add number to the array\n"
        b"2) Add random number to the array\n"
        b"3) Sum numbers\n"
        b"4) Exit\n"
        b"  Index: \n"
        b"1) Add number to the array\n"
        b"2) Add random number to the array\n"
        b"3) Sum numbers\n"
        b"4) Exit\n"
    )


def test_inspect():
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "i386", "uc_stop"), auto_load_libs=False)

    def main_state(argc, add_options=None):
        add_options = add_options or so.unicorn
        main_addr = p.loader.find_symbol("main").rebased_addr
        return p.factory.call_state(main_addr, argc, angr.PointerWrapper([]), add_options=add_options)

    # test breaking on specific addresses
    s_break_addr = main_state(1)
    addr0 = 0x08048479  # at the beginning of a basic block, at end of stop_normal function
    addr1 = 0x080485D0  # this is at the beginning of main, in the middle of a basic block
    addr2 = 0x08048461  # another non-bb address, at the start of stop_normal
    addr3 = 0x0804847C  # address of a block that should not get hit (stop_symbolc function)
    addr4 = 0x08048632  # another address that shouldn't get hit, near end of main
    hits = {addr0: 0, addr1: 0, addr2: 0, addr3: 0, addr4: 0}

    def create_addr_action(addr):
        def action(_state):
            hits[addr] += 1

        return action

    for addr in [addr0, addr1, addr2]:
        s_break_addr.inspect.b("instruction", instruction=addr, action=create_addr_action(addr))

    pg_instruction = p.factory.simulation_manager(s_break_addr)
    pg_instruction.run()
    assert hits[addr0] == 1
    assert hits[addr1] == 1
    assert hits[addr2] == 1
    assert hits[addr3] == 0
    assert hits[addr4] == 0

    # test breaking on every instruction
    def collect_trace(options):
        s_break_every = main_state(1, add_options=options)
        trace = []

        def action_every(state):
            trace.append(state.addr)

        s_break_every.inspect.b("instruction", action=action_every)
        pg_break_every = p.factory.simulation_manager(s_break_every)
        pg_break_every.run()

    assert collect_trace(so.unicorn) == collect_trace(set())


def test_explore():
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "i386", "uc_stop"), auto_load_libs=False)

    def main_state(argc, add_options=None):
        add_options = add_options or so.unicorn
        main_addr = p.loader.find_symbol("main").rebased_addr
        return p.factory.call_state(main_addr, argc, angr.PointerWrapper([]), add_options=add_options)

    addr = 0x08048479
    s_explore = main_state(1)
    pg_explore_find = p.factory.simulation_manager(s_explore)
    pg_explore_find.explore(find=addr)
    assert len(pg_explore_find.found) == 1
    assert pg_explore_find.found[0].addr == addr

    pg_explore_avoid = p.factory.simulation_manager(s_explore)
    pg_explore_avoid.explore(avoid=addr)
    assert len(pg_explore_avoid.avoid) == 1
    assert pg_explore_avoid.avoid[0].addr == addr


def test_single_step():
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "i386", "uc_stop"), auto_load_libs=False)

    def main_state(argc, add_options=None):
        add_options = add_options or so.unicorn
        main_addr = p.loader.find_symbol("main").rebased_addr
        return p.factory.call_state(main_addr, argc, angr.PointerWrapper([]), add_options=add_options)

    s_main = main_state(1)

    step1 = s_main.block().instruction_addrs[1]
    successors1 = s_main.step(num_inst=1).successors
    assert len(successors1) == 1
    assert successors1[0].addr == step1

    step5 = s_main.block().instruction_addrs[5]
    successors2 = successors1[0].step(num_inst=4).successors
    assert len(successors2) == 1
    assert successors2[0].addr == step5


def test_symbolic_flags_preserved_on_stop():
    """
    Test if symbolic flags are preserved when unicorn engine stops. This is needed for cases where compare is performed
    in one block and conditional jump in another.
    """

    p = angr.Project(os.path.join(test_location, "binaries", "tests", "x86_64", "test_symbolic_flags_in_unicorn"))
    init_state = p.factory.full_init_state(add_options=angr.options.unicorn)
    simgr = p.factory.simgr(init_state)
    simgr.run()
    result = None
    for final_state in simgr.deadended:
        if b"Congrats" in final_state.posix.dumps(1):
            result = final_state.posix.dumps(0)
            break

    assert result == b"FLAG{l00ps_4r3_t00_34sy_r1gh7??}"


if __name__ == "__main__":
    import logging

    logging.getLogger("angr.state_plugins.unicorn_engine").setLevel("DEBUG")
    logging.getLogger("angr.engines.unicorn_engine").setLevel("INFO")
    logging.getLogger("angr.factory").setLevel("DEBUG")
    logging.getLogger("angr.project").setLevel("DEBUG")
    # logging.getLogger('claripy.backends.backend_z3').setLevel('DEBUG')

    import sys

    if len(sys.argv) > 1:
        for arg in sys.argv[1:]:
            print("test_" + arg)
            res = globals()["test_" + arg]()
            if hasattr(res, "__iter__"):
                for ft in res:
                    fo = ft[0]
                    fa = ft[1:]
                    print("...", fa)
                    fo(*fa)
    else:
        for fk, fv in list(globals().items()):
            if fk.startswith("test_") and callable(fv):
                print(fk)
                res = fv()
                if hasattr(res, "__iter__"):
                    for ft in res:
                        fo = ft[0]
                        fa = ft[1:]
                        print("...", fa)
                        fo(*fa)



import sys


from angr import SimState, SimFile


def test_file_create():
    # Create a state first
    state = SimState(arch="AMD64", mode="symbolic")

    # Create a file
    fd = state.posix.open(b"test", 1)

    assert fd == 3


def test_file_read():
    state = SimState(arch="AMD64", mode="symbolic")

    content = state.solver.BVV(0xBADF00D, 32)
    content_size = content.size() // 8

    fd = state.posix.open(b"test", 1)
    simfd = state.posix.get_fd(fd)
    simfd.write_data(content)
    simfd.seek(0)
    simfd.read(0xC0000000, content_size)

    data = state.memory.load(0xC0000000, content_size)

    assert data is content


def test_file_seek():
    # TODO: Make this test more complete

    state = SimState(arch="AMD64", mode="symbolic")

    # Normal seeking
    fd = state.posix.open(b"test1", 1)
    simfd = state.posix.get_fd(fd)
    simfd.seek(0, "start")
    assert state.solver.is_true(simfd.tell() == 0)
    state.posix.close(fd)

    # TODO: test case: seek cannot go beyond the file size or current file pos

    # seek should not work for stdin/stdout/stderr
    assert state.solver.is_false(state.posix.get_fd(0).seek(0))
    assert state.solver.is_false(state.posix.get_fd(1).seek(0))
    assert state.solver.is_false(state.posix.get_fd(2).seek(0))

    # Seek from the end
    state.fs.insert("test2", SimFile(name="qwer", size=20))
    fd = state.posix.open(b"test2", 1)
    simfd = state.posix.get_fd(fd)
    simfd.seek(0, "end")
    assert state.solver.is_true(simfd.tell() == 20)
    state.posix.close(fd)

    # seek to a symbolic position (whence symbolic end)
    fd = state.posix.open(b"unknown_size", 1)
    simfd = state.posix.get_fd(fd)
    real_end = state.fs.get("unknown_size").size
    simfd.seek(0, "end")
    assert real_end is simfd.tell()
    state.posix.close(fd)


def main():
    g = globals()
    if len(sys.argv) > 1:
        f = "test_" + sys.argv[1]
        if f in g and hasattr(g[f], "__call__"):
            g[f]()
    else:
        for f, func in g.items():
            if f.startswith("test_") and hasattr(func, "__call__"):
                func()


if __name__ == "__main__":
    main()



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("api-ms-win-appmodel-runtime-l1-1-3.dll")
prototypes = \
    {
        #
        'GetPackagePathByFullName2': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="PackagePathType"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["packageFullName", "packagePathType", "pathLength", "path"]),
        #
        'GetStagedPackagePathByFullName2': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="PackagePathType"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["packageFullName", "packagePathType", "pathLength", "path"]),
        #
        'GetCurrentPackageInfo2': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="PackagePathType"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["flags", "packagePathType", "bufferLength", "buffer", "count"]),
        #
        'GetCurrentPackagePath2': SimTypeFunction([SimTypeInt(signed=False, label="PackagePathType"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["packagePathType", "pathLength", "path"]),
        #
        'GetPackageInfo2': SimTypeFunction([SimTypePointer(SimStruct({"reserved": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="_PACKAGE_INFO_REFERENCE", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="PackagePathType"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["packageInfoReference", "flags", "packagePathType", "bufferLength", "buffer", "count"]),
    }

lib.set_prototypes(prototypes)



import angr

######################################
# openlog
######################################


class closelog(angr.SimProcedure):
    # pylint:disable=arguments-differ

    def run(self):
        # A stub for closelog that does not do anything yet.
        return



import collections.abc
import claripy
from typing import Optional, TYPE_CHECKING

from .protos import variables_pb2 as pb2
from .serializable import Serializable

if TYPE_CHECKING:
    import archinfo


class SimVariable(Serializable):
    __slots__ = [
        "ident",
        "name",
        "region",
        "category",
        "renamed",
        "candidate_names",
        "size",
    ]

    def __init__(self, ident=None, name=None, region: Optional[int] = None, category=None, size: Optional[int] = None):
        """
        :param ident: A unique identifier provided by user or the program. Usually a string.
        :param str name: Name of this variable.
        """
        self.ident = ident
        self.name = name
        self.region: Optional[int] = region
        self.category: Optional[str] = category
        self.renamed = False
        self.candidate_names = None
        self.size = size

    def copy(self):
        raise NotImplementedError()

    def loc_repr(self, arch: "archinfo.Arch"):
        """
        The representation that shows up in a GUI
        """
        raise NotImplementedError()

    def _set_base(self, obj):
        obj.base.ident = self.ident
        if self.category is not None:
            obj.base.category = self.category
        if self.region is not None:
            obj.base.region = self.region
        if self.name is not None:
            obj.base.name = self.name
        obj.base.renamed = self.renamed

    def _from_base(self, obj):
        self.ident = obj.base.ident
        if obj.base.HasField("category"):
            self.category = obj.base.category
        else:
            self.category = None
        if obj.base.HasField("region"):
            self.region = obj.base.region
        self.name = obj.base.name
        self.renamed = obj.base.renamed

    @property
    def is_function_argument(self):
        return self.ident and self.ident.startswith("arg_")

    #
    # Operations
    #

    def __add__(self, other):
        if isinstance(other, int) and other == 0:
            return self
        return None

    def __sub__(self, other):
        if isinstance(other, int) and other == 0:
            return self
        return None


class SimConstantVariable(SimVariable):
    __slots__ = ["value", "_hash"]

    def __init__(self, ident=None, value=None, region=None, size=None):
        super().__init__(ident=ident, region=region, size=size)
        self.value = value
        self._hash = None

    def __repr__(self):
        s = f"<{self.region}|const {self.value}>"

        return s

    def loc_repr(self, arch):
        return f"const {self.value}"

    def __eq__(self, other):
        if not isinstance(other, SimConstantVariable):
            return False

        if self.value is None or other.value is None:
            # they may or may not represent the same constant. return not equal to be safe
            return False

        return self.ident == other.ident and self.value == other.value and self.region == other.region

    def __hash__(self):
        if self._hash is None:
            self._hash = hash(("const", self.value, self.ident, self.region, self.ident))
        return self._hash

    def copy(self) -> "SimConstantVariable":
        r = SimConstantVariable(ident=self.ident, value=self.value, region=self.region, size=self.size)
        r._hash = self._hash
        return r


class SimTemporaryVariable(SimVariable):
    __slots__ = ["tmp_id", "_hash"]

    def __init__(self, tmp_id, size=None):
        SimVariable.__init__(self, size=size)

        self.tmp_id = tmp_id
        self._hash = None

    def __repr__(self):
        s = "<tmp %d>" % (self.tmp_id,)
        return s

    def loc_repr(self, arch):
        return f"tmp #{self.tmp_id}"

    def __hash__(self):
        if self._hash is None:
            self._hash = hash("tmp_%d" % (self.tmp_id))
        return self._hash

    def __eq__(self, other):
        if isinstance(other, SimTemporaryVariable):
            return hash(self) == hash(other)

        return False

    def copy(self) -> "SimTemporaryVariable":
        r = SimTemporaryVariable(self.tmp_id, size=self.size)
        r._hash = self._hash
        return r

    @classmethod
    def _get_cmsg(cls):
        return pb2.TemporaryVariable()

    def serialize_to_cmessage(self):
        obj = self._get_cmsg()
        self._set_base(obj)
        obj.tmp_id = self.tmp_id
        return obj

    @classmethod
    def parse_from_cmessage(cls, cmsg, **kwargs):
        obj = cls(cmsg.tmp_id)
        obj._from_base(cmsg)
        return obj


class SimRegisterVariable(SimVariable):
    __slots__ = ["reg", "_hash"]

    def __init__(self, reg_offset, size, ident=None, name=None, region=None, category=None):
        SimVariable.__init__(self, ident=ident, name=name, region=region, category=category, size=size)

        self.reg: int = reg_offset
        self._hash: Optional[int] = None

    @property
    def bits(self):
        return self.size * 8

    def __repr__(self):
        ident_str = "[%s]" % self.ident if self.ident else ""
        region_str = hex(self.region) if isinstance(self.region, int) else self.region

        s = f"<{region_str}{ident_str}|Reg {self.reg}, {self.size}B>"

        return s

    def loc_repr(self, arch):
        return arch.translate_register_name(self.reg, self.size)

    def __hash__(self):
        if self._hash is None:
            self._hash = hash(("reg", self.region, self.reg, self.size, self.ident))
        return self._hash

    def __eq__(self, other):
        if isinstance(other, SimRegisterVariable):
            return (
                self.ident == other.ident
                and self.reg == other.reg
                and self.size == other.size
                and self.region == other.region
            )

        return False

    def copy(self) -> "SimRegisterVariable":
        s = SimRegisterVariable(
            self.reg, self.size, ident=self.ident, name=self.name, region=self.region, category=self.category
        )
        s._hash = self._hash
        return s

    @classmethod
    def _get_cmsg(cls):
        return pb2.RegisterVariable()

    def serialize_to_cmessage(self):
        obj = self._get_cmsg()
        self._set_base(obj)
        obj.reg = self.reg
        obj.size = self.size
        return obj

    @classmethod
    def parse_from_cmessage(cls, cmsg, **kwargs):
        obj = cls(
            cmsg.reg,
            cmsg.size,
        )
        obj._from_base(cmsg)
        return obj


class SimMemoryVariable(SimVariable):
    __slots__ = ["addr", "_hash"]

    def __init__(self, addr, size, ident=None, name=None, region=None, category=None):
        SimVariable.__init__(self, ident=ident, name=name, region=region, category=category, size=size)

        self.addr = addr

        if isinstance(size, claripy.ast.BV) and not size.symbolic:
            # Convert it to a concrete number
            size = size._model_concrete.value

        self.size = size
        self._hash = None

    def __repr__(self):
        if type(self.size) is int:
            size = "%d" % self.size
        else:
            size = "%s" % self.size

        if type(self.addr) is int:
            s = f"<{self.name}: {self.region}-Mem {self.addr:#x} {size}>"
        else:
            s = f"<{self.name}: {self.region}-Mem {self.addr} {size}>"

        return s

    def loc_repr(self, arch):
        return f"[{self.addr:#x}]"

    def __hash__(self):
        if self._hash is not None:
            return self._hash

        if isinstance(self.addr, AddressWrapper):
            addr_hash = hash(self.addr)
        elif type(self.addr) is int:
            addr_hash = self.addr
        elif self.addr._model_concrete is not self.addr:
            addr_hash = hash(self.addr._model_concrete)
        elif self.addr._model_vsa is not self.addr:
            addr_hash = hash(self.addr._model_vsa)
        elif self.addr._model_z3 is not self.addr:
            addr_hash = hash(self.addr._model_z3)
        else:
            addr_hash = hash(self.addr)
        self._hash = hash((addr_hash, hash(self.size), self.ident))

        return self._hash

    def __eq__(self, other):
        if isinstance(other, SimMemoryVariable):
            return self.ident == other.ident and self.addr == other.addr and self.size == other.size

        return False

    @property
    def bits(self):
        return self.size * 8

    def copy(self) -> "SimMemoryVariable":
        r = SimMemoryVariable(
            self.addr, self.size, ident=self.ident, name=self.name, region=self.region, category=self.category
        )
        r._hash = self._hash
        return r

    @classmethod
    def _get_cmsg(cls):
        return pb2.MemoryVariable()

    def serialize_to_cmessage(self):
        obj = self._get_cmsg()
        self._set_base(obj)
        obj.addr = self.addr
        obj.size = self.size
        return obj

    @classmethod
    def parse_from_cmessage(cls, cmsg, **kwargs):
        obj = cls(
            cmsg.addr,
            cmsg.size,
        )
        obj._from_base(cmsg)
        return obj


class SimStackVariable(SimMemoryVariable):
    __slots__ = (
        "base",
        "offset",
        "base_addr",
    )

    def __init__(self, offset, size, base="sp", base_addr=None, ident=None, name=None, region=None, category=None):
        if isinstance(offset, int) and offset > 0x1000000:
            # I don't think any positive stack offset will be greater than that...
            # convert it to a negative number
            mask = (1 << offset.bit_length()) - 1
            offset = -((0 - offset) & mask)

        if base_addr is not None:
            addr = offset + base_addr
        else:
            # TODO: this is not optimal
            addr = offset

        super().__init__(addr, size, ident=ident, name=name, region=region, category=category)

        self.base = base
        self.offset = offset
        self.base_addr = base_addr

    def __repr__(self):
        if type(self.size) is int:
            size = "%d" % self.size
        else:
            size = "%s" % self.size

        prefix = "%s(stack)" % self.name if self.name is not None else "Stack"
        ident = "[%s]" % self.ident if self.ident else ""
        region_str = hex(self.region) if isinstance(self.region, int) else self.region

        if type(self.offset) is int:
            if self.offset < 0:
                offset = "%#x" % self.offset
            elif self.offset > 0:
                offset = "+%#x" % self.offset
            else:
                offset = ""

            s = f"<{region_str}{ident}|{prefix} {self.base}{offset}, {size} B>"
        else:
            s = f"<{region_str}{ident}|{prefix} {self.base}{self.addr}, {size} B>"

        return s

    def loc_repr(self, arch):
        return f"[{self.base}{self.offset:+#x}]"

    def __eq__(self, other):
        if type(other) is not SimStackVariable:
            return False

        return (
            self.ident == other.ident
            and self.base == other.base
            and self.offset == other.offset
            and self.size == other.size
        )

    def __hash__(self):
        return hash((self.ident, self.base, self.offset, self.size))

    def copy(self) -> "SimStackVariable":
        s = SimStackVariable(
            self.offset,
            self.size,
            base=self.base,
            base_addr=self.base_addr,
            ident=self.ident,
            name=self.name,
            region=self.region,
            category=self.category,
        )
        s._hash = self._hash
        return s

    @classmethod
    def _get_cmsg(cls):
        return pb2.StackVariable()

    def serialize_to_cmessage(self):
        obj = self._get_cmsg()
        self._set_base(obj)
        obj.sp_base = self.base == "sp"
        obj.offset = self.offset
        obj.size = self.size
        return obj

    @classmethod
    def parse_from_cmessage(cls, cmsg, **kwargs):
        obj = cls(
            cmsg.offset,
            cmsg.size,
            base="sp" if cmsg.sp_base else "bp",
        )
        obj._from_base(cmsg)
        return obj


class SimVariableSet(collections.abc.MutableSet):
    """
    A collection of SimVariables.
    """

    def __init__(self):
        self.register_variables = set()
        # For the sake of performance optimization, all elements in register_variables must be concrete integers which
        # representing register offsets..
        # There shouldn't be any problem apart from GetI/PutI instructions. We simply ignore them for now.
        # TODO: Take care of register offsets that are not aligned to (arch.bytes)
        # TODO: arch.bits/what? That number has no power here anymore.
        self.register_variable_offsets = set()

        # memory_variables holds SimMemoryVariable objects
        self.memory_variables = set()
        # For the sake of performance, we have another set that stores memory addresses of memory_variables
        self.memory_variable_addresses = set()

    def add(self, item):  # pylint:disable=arguments-differ
        if type(item) is SimRegisterVariable:
            if not self.contains_register_variable(item):
                self.add_register_variable(item)
        elif type(item) is SimMemoryVariable:
            if not self.contains_memory_variable(item):
                self.add_memory_variable(item)
        else:
            # TODO:
            raise Exception("WTF")

    def add_register_variable(self, reg_var):
        self.register_variables.add(reg_var)
        self.register_variable_offsets.add(reg_var.reg)

    def add_memory_variable(self, mem_var):
        self.memory_variables.add(mem_var)
        base_address = mem_var.addr.address  # Dealing with AddressWrapper
        for i in range(mem_var.size):
            self.memory_variable_addresses.add(base_address + i)

    def discard(self, item):  # pylint:disable=arguments-differ
        if type(item) is SimRegisterVariable:
            if self.contains_register_variable(item):
                self.discard_register_variable(item)
        elif isinstance(item, SimMemoryVariable):
            if self.contains_memory_variable(item):
                self.discard_memory_variable(item)
        else:
            # TODO:
            raise Exception("")

    def discard_register_variable(self, reg_var):
        self.register_variables.remove(reg_var)
        self.register_variable_offsets.remove(reg_var.reg)

    def discard_memory_variable(self, mem_var):
        self.memory_variables.remove(mem_var)
        for i in range(mem_var.size):
            self.memory_variable_addresses.remove(mem_var.addr.address + i)

    def __len__(self):
        return len(self.register_variables) + len(self.memory_variables)

    def __iter__(self):
        yield from self.register_variables
        yield from self.memory_variables

    def add_memory_variables(self, addrs, size):
        for a in addrs:
            var = SimMemoryVariable(a, size)
            self.add_memory_variable(var)

    def copy(self):
        s = SimVariableSet()
        s.register_variables |= self.register_variables
        s.register_variable_offsets |= self.register_variable_offsets
        s.memory_variables |= self.memory_variables
        s.memory_variable_addresses |= self.memory_variable_addresses

        return s

    def complement(self, other):
        """
        Calculate the complement of `self` and `other`.

        :param other:   Another SimVariableSet instance.
        :return:        The complement result.
        """

        s = SimVariableSet()
        s.register_variables = self.register_variables - other.register_variables
        s.register_variable_offsets = self.register_variable_offsets - other.register_variable_offsets
        s.memory_variables = self.memory_variables - other.memory_variables
        s.memory_variable_addresses = self.memory_variable_addresses - other.memory_variable_addresses

        return s

    def contains_register_variable(self, reg_var):
        reg_offset = reg_var.reg
        # TODO: Make sure reg_offset is aligned to machine-word length

        return reg_offset in self.register_variable_offsets

    def contains_memory_variable(self, mem_var):
        a = mem_var.addr
        if type(a) in (tuple, list):
            a = a[-1]

        return a in self.memory_variable_addresses

    def __ior__(self, other):
        # other must be a SimVariableSet
        self.register_variables |= other.register_variables
        self.register_variable_offsets |= other.register_variable_offsets
        self.memory_variables |= other.memory_variables
        self.memory_variable_addresses |= other.memory_variable_addresses

    def __contains__(self, item):
        if type(item) is SimRegisterVariable:
            return self.contains_register_variable(item)

        elif type(item) is SimMemoryVariable:
            # TODO: Make it better!
            return self.contains_memory_variable(item)

        else:
            __import__("ipdb").set_trace()
            raise Exception("WTF is this variable?")


from .storage.memory_mixins.regioned_memory.region_data import AddressWrapper



import logging

import claripy
from cle import BackedCGC

from ..procedures import SIM_LIBRARIES as L
from ..state_plugins import SimActionData
from .. import sim_options as o
from .userland import SimUserland

_l = logging.getLogger(name=__name__)


class SimCGC(SimUserland):
    """
    Environment configuration for the CGC DECREE platform
    """

    def __init__(self, project, **kwargs):
        super().__init__(project, syscall_library=L["cgcabi"], syscall_addr_alignment=1, name="CGC", **kwargs)

    # pylint: disable=arguments-differ
    def state_blank(self, flag_page=None, allocate_stack_page_count=0x100, **kwargs):
        """
        :param flag_page:                   Flag page content, either a string or a list of BV8s
        :param allocate_stack_page_count:   Number of pages to pre-allocate for stack
        """
        # default stack as specified in the cgc abi
        if kwargs.get("stack_end", None) is None:
            kwargs["stack_end"] = 0xBAAAB000
        if kwargs.get("stack_size", None) is None:
            kwargs["stack_size"] = 1024 * 1024 * 8

        s = super().state_blank(**kwargs)  # pylint:disable=invalid-name

        # pre-grow the stack. unsure if this is strictly required or just a hack around a compiler bug
        if hasattr(s.memory, "allocate_stack_pages"):
            s.memory.allocate_stack_pages(kwargs["stack_end"] - 1, allocate_stack_page_count * 0x1000)

        # Map the flag page
        if o.ABSTRACT_MEMORY not in s.options:
            s.memory.map_region(0x4347C000, 4096, 1)

        # Create the CGC plugin
        s.get_plugin("cgc")

        # Set maximum bytes a single receive syscall should read
        s.cgc.max_receive_size = kwargs.get("cgc_max_recv_size", 0)

        # Set up the flag page
        if flag_page is None:
            flag_page = [s.solver.BVS("cgc-flag-byte-%d" % i, 8, key=("flag", i), eternal=True) for i in range(0x1000)]
        elif type(flag_page) is bytes:
            flag_page = [s.solver.BVV(c, 8) for c in flag_page]
        elif type(flag_page) is list:
            pass
        else:
            raise ValueError("Bad flag page: expected None, bytestring, or list, but got %s" % type(flag_page))

        s.cgc.flag_bytes = flag_page
        if s.mode != "static":
            s.memory.store(0x4347C000, claripy.Concat(*s.cgc.flag_bytes), priv=True)

        # set up the address for concrete transmits and receive
        s.unicorn.cgc_transmit_addr = self.syscall_from_number(2).addr
        s.unicorn.cgc_receive_addr = self.syscall_from_number(3).addr
        s.unicorn.cgc_random_addr = self.syscall_from_number(7).addr

        s.libc.max_str_len = 1000000
        s.libc.max_strtol_len = 10
        s.libc.max_memcpy_size = 0x100000
        s.libc.max_buffer_size = 0x100000

        return s

    def state_entry(self, add_options=None, **kwargs):
        if isinstance(self.project.loader.main_object, BackedCGC):
            kwargs["permissions_backer"] = (True, self.project.loader.main_object.permissions_map)
        if add_options is None:
            add_options = set()
        add_options.add(o.ZERO_FILL_UNCONSTRAINED_MEMORY)

        state = super().state_entry(add_options=add_options, **kwargs)

        if isinstance(self.project.loader.main_object, BackedCGC):
            # Update allocation base
            state.cgc.allocation_base = self.project.loader.main_object.current_allocation_base

            # Do all the writes
            writes_backer = self.project.loader.main_object.writes_backer
            stdout = state.posix.get_fd(1)
            pos = 0
            for size in writes_backer:
                if size == 0:
                    continue
                str_to_write = state.solver.BVS("file_write", size * 8)
                a = SimActionData(
                    state, "file_1_0", "write", addr=claripy.BVV(pos, state.arch.bits), data=str_to_write, size=size
                )
                stdout.write_data(str_to_write)
                state.history.add_action(a)
                pos += size

        else:
            # Set CGC-specific variables
            state.regs.eax = 0
            state.regs.ebx = 0
            state.regs.ecx = 0x4347C000
            state.regs.edx = 0
            state.regs.edi = 0
            state.regs.esi = 0
            state.regs.esp = 0xBAAAAFFC
            state.regs.ebp = 0
            state.regs.cc_dep1 = 0x202  # default eflags
            state.regs.cc_op = 0  # OP_COPY
            state.regs.cc_dep2 = 0  # doesn't matter
            state.regs.cc_ndep = 0  # doesn't matter

            # fpu values
            state.regs.mm0 = 0
            state.regs.mm1 = 0
            state.regs.mm2 = 0
            state.regs.mm3 = 0
            state.regs.mm4 = 0
            state.regs.mm5 = 0
            state.regs.mm6 = 0
            state.regs.mm7 = 0
            state.regs.fpu_tags = 0
            state.regs.fpround = 0
            state.regs.fc3210 = 0x0300
            state.regs.ftop = 0

            # sse values
            state.regs.sseround = 0
            state.regs.xmm0 = 0
            state.regs.xmm1 = 0
            state.regs.xmm2 = 0
            state.regs.xmm3 = 0
            state.regs.xmm4 = 0
            state.regs.xmm5 = 0
            state.regs.xmm6 = 0
            state.regs.xmm7 = 0

            # segmentation registers
            state.regs.ds = 0
            state.regs.es = 0
            state.regs.fs = 0
            state.regs.gs = 0
            state.regs.ss = 0
            state.regs.cs = 0

        return state



import sys
import random
import angr

from angr import SimState, SIM_LIBRARIES

FAKE_ADDR = 0x100000


def make_func(name):
    return (
        lambda state, arguments: SIM_LIBRARIES["libc.so.6"]
        .get(name, "AMD64")
        .execute(state, arguments=arguments)
        .ret_expr
    )


strstr = make_func("strstr")
strtok_r = make_func("strtok_r")
strcmp = make_func("strcmp")
strchr = make_func("strchr")
strncmp = make_func("strncmp")
strlen = make_func("strlen")
strncpy = make_func("strncpy")
strcpy = make_func("strcpy")
memset = make_func("memset")
memcpy = make_func("memcpy")
memcmp = make_func("memcmp")
sprintf = make_func("sprintf")
getc = make_func("_IO_getc")
fgetc = make_func("fgetc")
getchar = make_func("getchar")
scanf = make_func("scanf")
wcscmp = make_func("wcscmp")

import logging

l = logging.getLogger("angr.tests.string")


def make_state_with_stdin(content):
    s = SimState(arch="AMD64", mode="symbolic")
    stdin_storage = angr.storage.file.SimFile("stdin", content=content)
    stdin = angr.storage.file.SimFileDescriptor(stdin_storage)
    s.register_plugin("posix", angr.state_plugins.SimSystemPosix(stdin=stdin_storage, fd={0: stdin}))
    return s


def test_inline_strlen():
    s = SimState(arch="AMD64", mode="symbolic")

    l.info("fully concrete string")
    a_str = s.solver.BVV(0x41414100, 32)
    a_addr = s.solver.BVV(0x10, 64)
    s.memory.store(a_addr, a_str, endness="Iend_BE")
    a_len = strlen(s, arguments=[a_addr])
    assert s.solver.unique(a_len)
    assert s.solver.eval(a_len) == 3

    l.info("concrete-terminated string")
    b_str = s.solver.Concat(s.solver.BVS("mystring", 24), s.solver.BVV(0, 8))
    b_addr = s.solver.BVV(0x20, 64)
    s.memory.store(b_addr, b_str, endness="Iend_BE")
    b_len = strlen(s, arguments=[b_addr])
    assert s.solver.max_int(b_len) == 3
    assert tuple(sorted(s.solver.eval_upto(b_len, 10))) == (0, 1, 2, 3)

    l.info("fully unconstrained")
    u_addr = s.solver.BVV(0x50, 64)
    u_len_sp = strlen(s, arguments=[u_addr])
    u_len = u_len_sp
    assert len(s.solver.eval_upto(u_len, 100)) == s.libc.buf_symbolic_bytes
    assert s.solver.max_int(u_len) == s.libc.buf_symbolic_bytes - 1
    # print u_len_sp.solver.maximum_null

    # s.add_constraints(u_len < 16)

    assert s.solver.eval_upto(s.memory.load(0x50 + u_len, 1), 300) == [0]
    #
    # This tests if a strlen can influence a symbolic str.
    #
    l.info("Trying to influence length.")
    s = SimState(arch="AMD64", mode="symbolic")
    str_c = s.solver.BVS("some_string", 8 * 16)
    c_addr = s.solver.BVV(0x10, 64)
    s.memory.store(c_addr, str_c, endness="Iend_BE")
    c_len = strlen(s, arguments=[c_addr])
    assert len(s.solver.eval_upto(c_len, 100)) == s.libc.buf_symbolic_bytes
    assert s.solver.max_int(c_len) == s.libc.buf_symbolic_bytes - 1

    one_s = s.copy()
    one_s.add_constraints(c_len == 1)
    assert one_s.solver.eval(str_c, cast_to=bytes).index(b"\x00") == 1
    str_test = one_s.memory.load(c_addr, 2, endness="Iend_BE")
    assert len(one_s.solver.eval_upto(str_test, 300, cast_to=bytes)) == 255

    for i in range(16):
        test_s = s.copy()
        test_s.add_constraints(c_len == i)
        str_test = test_s.memory.load(c_addr, i + 1, endness="Iend_BE")
        assert test_s.solver.eval(str_test, cast_to=bytes).index(b"\x00") == i
        for j in range(i):
            assert not test_s.solver.unique(test_s.memory.load(c_addr + j, 1))


def test_inline_strcmp():
    s = SimState(arch="AMD64", mode="symbolic")
    str_a = s.solver.BVV(0x41414100, 32)
    str_b = s.solver.BVS("mystring", 32)

    a_addr = s.solver.BVV(0x10, 64)
    b_addr = s.solver.BVV(0xB0, 64)

    s.memory.store(a_addr, str_a, endness="Iend_BE")
    s.memory.store(b_addr, str_b, endness="Iend_BE")

    s_cmp = s.copy()
    cmpres = strcmp(s_cmp, arguments=[a_addr, b_addr])
    s_match = s_cmp.copy()
    s_nomatch = s_cmp.copy()
    s_match.add_constraints(cmpres == 0)
    s_nomatch.add_constraints(cmpres != 0)

    assert s_match.solver.unique(str_b)
    assert not s_nomatch.solver.unique(str_b)
    assert s_match.solver.eval(str_b, cast_to=bytes) == b"AAA\x00"

    s_ncmp = s.copy()
    ncmpres = strncmp(s_ncmp, arguments=[a_addr, b_addr, s.solver.BVV(2, s.arch.bits)])
    s_match = s_ncmp.copy()
    s_nomatch = s_ncmp.copy()
    s_match.add_constraints(ncmpres == 0)
    s_nomatch.add_constraints(ncmpres != 0)

    assert not s_match.solver.unique(str_b)
    assert s_match.solver.unique(s_match.memory.load(b_addr, 2))
    assert len(s_match.solver.eval_upto(s_match.memory.load(b_addr, 3), 300)) == 256
    assert not s_nomatch.solver.unique(str_b)

    l.info("concrete a, symbolic b")
    s = SimState(arch="AMD64", mode="symbolic")
    str_a = s.solver.BVV(0x41424300, 32)
    str_b = s.solver.BVS("mystring", 32)
    a_addr = s.solver.BVV(0x10, 64)
    b_addr = s.solver.BVV(0xB0, 64)
    s.memory.store(a_addr, str_a, endness="Iend_BE")
    s.memory.store(b_addr, str_b, endness="Iend_BE")

    s_cmp = s.copy()
    cmpres = strncmp(s_cmp, arguments=[a_addr, b_addr, s.solver.BVV(2, s_cmp.arch.bits)])
    s_match = s_cmp.copy()
    s_nomatch = s_cmp.copy()
    s_match.add_constraints(cmpres == 0)
    s_nomatch.add_constraints(cmpres != 0)

    assert s_match.solver.solution(str_b, 0x41420000)
    assert s_match.solver.solution(str_b, 0x41421234)
    assert s_match.solver.solution(str_b, 0x41424300)
    assert not s_nomatch.solver.solution(str_b, 0x41420000)
    assert not s_nomatch.solver.solution(str_b, 0x41421234)
    assert not s_nomatch.solver.solution(str_b, 0x41424300)

    l.info("symbolic a, symbolic b")
    s = SimState(arch="AMD64", mode="symbolic")
    a_addr = s.solver.BVV(0x10, 64)
    b_addr = s.solver.BVV(0xB0, 64)

    s_cmp = s.copy()
    cmpres = strcmp(s_cmp, arguments=[a_addr, b_addr])
    s_match = s_cmp.copy()
    s_nomatch = s_cmp.copy()
    s_match.add_constraints(cmpres == 0)
    s_nomatch.add_constraints(cmpres != 0)

    m_res = strcmp(s_match, arguments=[a_addr, b_addr])
    s_match.add_constraints(m_res != 0)
    nm_res = strcmp(s_nomatch, arguments=[a_addr, b_addr])
    s_nomatch.add_constraints(nm_res == 0)

    assert not s_match.satisfiable()
    assert not s_match.satisfiable()


def test_inline_strncmp():
    l.info("symbolic left, symbolic right, symbolic len")
    s = SimState(arch="AMD64", mode="symbolic")
    left = s.solver.BVS("left", 32)
    left_addr = s.solver.BVV(0x1000, 64)
    right = s.solver.BVS("right", 32)
    right_addr = s.solver.BVV(0x2000, 64)
    maxlen = s.solver.BVS("len", 64)

    s.memory.store(left_addr, left)
    s.memory.store(right_addr, right)

    s.add_constraints(strlen(s, arguments=[left_addr]) == 3)
    s.add_constraints(strlen(s, arguments=[right_addr]) == 0)

    s.add_constraints(maxlen != 0)
    c = strncmp(s, arguments=[left_addr, right_addr, maxlen])

    s_match = s.copy()
    s_match.add_constraints(c == 0)
    assert not s_match.satisfiable()
    # assert s_match.solver.min_int(maxlen) == 3

    s_nomatch = s.copy()
    s_nomatch.add_constraints(c != 0)
    assert s_nomatch.satisfiable()
    # assert s_nomatch.solver.max_int(maxlen) == 2

    l.info("zero-length")
    s = SimState(arch="AMD64", mode="symbolic")
    left = s.solver.BVS("left", 32)
    left_addr = s.solver.BVV(0x1000, 64)
    right = s.solver.BVS("right", 32)
    right_addr = s.solver.BVV(0x2000, 64)
    maxlen = s.solver.BVS("len", 64)
    left_len = strlen(s, arguments=[left_addr])
    right_len = strlen(s, arguments=[right_addr])
    c = strncmp(s, arguments=[left_addr, right_addr, maxlen])

    s.add_constraints(right_len == 0)
    s.add_constraints(left_len == 0)
    # s.add_constraints(c == 0)
    s.add_constraints(maxlen == 0)
    assert s.satisfiable()


def broken_inline_strstr():
    l.info("concrete haystack and needle")
    s = SimState(arch="AMD64", mode="symbolic")
    str_haystack = s.solver.BVV(0x41424300, 32)
    str_needle = s.solver.BVV(0x42430000, 32)
    addr_haystack = s.solver.BVV(0x10, 64)
    addr_needle = s.solver.BVV(0xB0, 64)
    s.memory.store(addr_haystack, str_haystack, endness="Iend_BE")
    s.memory.store(addr_needle, str_needle, endness="Iend_BE")

    ss_res = strstr(s, arguments=[addr_haystack, addr_needle])
    assert s.solver.unique(ss_res)
    assert s.solver.eval(ss_res) == 0x11

    l.info("concrete haystack, symbolic needle")
    s = SimState(arch="AMD64", mode="symbolic")
    str_haystack = s.solver.BVV(0x41424300, 32)
    str_needle = s.solver.BVS("wtf", 32)
    addr_haystack = s.solver.BVV(0x10, 64)
    addr_needle = s.solver.BVV(0xB0, 64)
    s.memory.store(addr_haystack, str_haystack, endness="Iend_BE")
    s.memory.store(addr_needle, str_needle, endness="Iend_BE")

    ss_res = strstr(s, arguments=[addr_haystack, addr_needle])
    assert not s.solver.unique(ss_res)
    assert len(s.solver.eval_upto(ss_res, 10)) == 4

    s_match = s.copy()
    s_nomatch = s.copy()
    s_match.add_constraints(ss_res != 0)
    s_nomatch.add_constraints(ss_res == 0)

    match_needle = str_needle[31:16]
    assert len(s_match.solver.eval_upto(match_needle, 300)) == 259
    assert len(s_match.solver.eval_upto(str_needle, 10)) == 10

    l.info("symbolic haystack, symbolic needle")
    s = SimState(arch="AMD64", mode="symbolic")
    s.libc.buf_symbolic_bytes = 5
    addr_haystack = s.solver.BVV(0x10, 64)
    addr_needle = s.solver.BVV(0xB0, 64)
    len_needle = strlen(s, arguments=[addr_needle])

    ss_res = strstr(s, arguments=[addr_haystack, addr_needle])
    assert not s.solver.unique(ss_res)
    assert len(s.solver.eval_upto(ss_res, 100)) == s.libc.buf_symbolic_bytes

    s_match = s.copy()
    s_nomatch = s.copy()
    s_match.add_constraints(ss_res != 0)
    s_nomatch.add_constraints(ss_res == 0)

    match_cmp = strncmp(s_match, arguments=[ss_res, addr_needle, len_needle])
    assert s_match.solver.eval_upto(match_cmp, 10) == [0]

    r_mm = strstr(s_match, arguments=[addr_haystack, addr_needle])
    s_match.add_constraints(r_mm == 0)
    assert not s_match.satisfiable()

    assert s_nomatch.satisfiable()
    s_nss = s_nomatch.copy()
    nomatch_ss = strstr(s_nss, arguments=[addr_haystack, addr_needle])
    s_nss.add_constraints(nomatch_ss != 0)
    assert not s_nss.satisfiable()


def test_strstr_inconsistency():
    l.info("symbolic haystack, symbolic needle")
    s = SimState(arch="AMD64", mode="symbolic")
    s.libc.buf_symbolic_bytes = 2
    addr_haystack = s.solver.BVV(0x10, 64)
    addr_needle = s.solver.BVV(0xB0, 64)
    # len_needle = strlen(s, inline=True, arguments=[addr_needle])

    ss_res = strstr(s, arguments=[addr_haystack, addr_needle])

    # slh_res = strlen(s, inline=True, arguments=[addr_haystack])
    # sln_res = strlen(s, inline=True, arguments=[addr_needle])
    # print "LENH:", s.solver.eval_upto(slh_res, 100)
    # print "LENN:", s.solver.eval_upto(sln_res, 100)

    assert not s.solver.unique(ss_res)
    assert sorted(s.solver.eval_upto(ss_res, 100)) == [0] + list(range(0x10, 0x10 + s.libc.buf_symbolic_bytes - 1))

    s.add_constraints(ss_res != 0)
    ss2 = strstr(s, arguments=[addr_haystack, addr_needle])
    s.add_constraints(ss2 == 0)
    assert not s.satisfiable()


def test_memcpy():
    l.info("concrete src, concrete dst, concrete len")
    l.debug("... full copy")
    s = SimState(arch="AMD64", mode="symbolic")
    dst = s.solver.BVV(0x41414141, 32)
    dst_addr = s.solver.BVV(0x1000, 64)
    src = s.solver.BVV(0x42424242, 32)
    src_addr = s.solver.BVV(0x2000, 64)

    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)
    memcpy(s, arguments=[dst_addr, src_addr, s.solver.BVV(4, 64)])
    new_dst = s.memory.load(dst_addr, 4, endness="Iend_BE")
    assert s.solver.eval_upto(new_dst, 2, cast_to=bytes) == [b"BBBB"]

    l.info("giant copy")
    s = SimState(arch="AMD64", mode="symbolic", remove_options=angr.options.simplification)
    s.memory._maximum_symbolic_size = 0x2000000
    size = s.solver.BVV(0x1000000, 64)
    data = s.solver.BVS("giant", 8 * 0x1_000_000)
    dst_addr = s.solver.BVV(0x2000000, 64)
    src_addr = s.solver.BVV(0x4000000, 64)
    s.memory.store(src_addr, data)

    memcpy(s, arguments=[dst_addr, src_addr, size])
    assert s.memory.load(dst_addr, size) is s.memory.load(src_addr, size)

    l.debug("... partial copy")
    s = SimState(arch="AMD64", mode="symbolic")
    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)
    memcpy(s, arguments=[dst_addr, src_addr, s.solver.BVV(2, 64)])
    new_dst = s.memory.load(dst_addr, 4, endness="Iend_BE")
    assert s.solver.eval_upto(new_dst, 2, cast_to=bytes) == [b"BBAA"]

    l.info("symbolic src, concrete dst, concrete len")
    s = SimState(arch="AMD64", mode="symbolic")
    dst = s.solver.BVV(0x41414141, 32)
    dst_addr = s.solver.BVV(0x1000, 64)
    src = s.solver.BVS("src", 32)
    src_addr = s.solver.BVV(0x2000, 64)

    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)

    # make sure it copies it all
    memcpy(s, arguments=[dst_addr, src_addr, s.solver.BVV(4, 64)])
    assert s.satisfiable()
    s.add_constraints(src != s.memory.load(dst_addr, 4))
    assert not s.satisfiable()

    l.info("symbolic src, concrete dst, symbolic len")
    s = SimState(arch="AMD64", mode="symbolic")
    dst = s.solver.BVV(0x41414141, 32)
    dst_addr = s.solver.BVV(0x1000, 64)
    src = s.solver.BVS("src", 32)
    src_addr = s.solver.BVV(0x2000, 64)
    cpylen = s.solver.BVS("len", 64)

    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)
    s.add_constraints(cpylen < 10)
    memcpy(s, arguments=[dst_addr, src_addr, cpylen])
    result = s.memory.load(dst_addr, 4, endness="Iend_BE")

    # make sure it copies it all
    s1 = s.copy()
    s1.add_constraints(cpylen == 1)
    assert s1.solver.unique(s1.memory.load(dst_addr + 1, 3))
    assert len(s1.solver.eval_upto(s1.memory.load(dst_addr, 1), 300)) == 256

    s2 = s.copy()
    s2.add_constraints(cpylen == 2)
    assert len(s2.solver.eval_upto(result[31:24], 300)) == 256
    assert len(s2.solver.eval_upto(result[23:16], 300)) == 256
    assert s2.solver.eval_upto(result[15:0], 300, cast_to=bytes) == [b"AA"]

    l.info("concrete src, concrete dst, symbolic len")
    dst = s2.solver.BVV(0x41414141, 32)
    dst_addr = s2.solver.BVV(0x1000, 64)
    src = s2.solver.BVV(0x42424242, 32)
    src_addr = s2.solver.BVV(0x2000, 64)

    s = SimState(arch="AMD64", mode="symbolic")
    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)
    cpylen = s.solver.BVS("len", 64)

    s.add_constraints(s.solver.ULE(cpylen, 4))
    memcpy(s, arguments=[dst_addr, src_addr, cpylen])
    new_dst = s.memory.load(dst_addr, 4, endness="Iend_BE")
    assert sorted(s.solver.eval_upto(new_dst, 300, cast_to=bytes)) == [b"AAAA", b"BAAA", b"BBAA", b"BBBA", b"BBBB"]


def test_memcmp():
    l.info("concrete src, concrete dst, concrete len")

    l.debug("... full cmp")
    s = SimState(arch="AMD64", mode="symbolic")
    dst = s.solver.BVV(0x41414141, 32)
    dst_addr = s.solver.BVV(0x1000, 64)
    src = s.solver.BVV(0x42424242, 32)
    src_addr = s.solver.BVV(0x2000, 64)
    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)
    r = memcmp(s, arguments=[dst_addr, src_addr, s.solver.BVV(4, 64)])
    assert s.satisfiable()

    s_pos = s.copy()
    s_pos.add_constraints(r.SGE(0))
    assert not s_pos.satisfiable()

    s_neg = s.copy()
    s_neg.add_constraints(r.SLT(0))
    assert s_neg.satisfiable()

    l.debug("... zero cmp")
    s = SimState(arch="AMD64", mode="symbolic")
    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)
    r = memcmp(s, arguments=[dst_addr, src_addr, s.solver.BVV(0, 64)])
    assert s.solver.eval_upto(r, 2) == [0]

    l.info("symbolic src, concrete dst, concrete len")
    s = SimState(arch="AMD64", mode="symbolic")
    dst = s.solver.BVV(0x41414141, 32)
    dst_addr = s.solver.BVV(0x1000, 64)
    src = s.solver.BVS("src", 32)

    src_addr = s.solver.BVV(0x2000, 64)

    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)

    # make sure it copies it all
    r = memcmp(s, arguments=[dst_addr, src_addr, s.solver.BVV(4, 64)])

    s_match = s.copy()
    s_match.add_constraints(r == 0)
    m = s_match.memory.load(src_addr, 4)
    assert s_match.solver.eval_upto(m, 2) == [0x41414141]

    s_nomatch = s.copy()
    s_nomatch.add_constraints(r != 0)
    m = s_nomatch.memory.load(src_addr, 4)
    assert not s_nomatch.solver.solution(m, 0x41414141)

    l.info("symbolic src, concrete dst, symbolic len")
    s = SimState(arch="AMD64", mode="symbolic")
    dst = s.solver.BVV(0x41414141, 32)
    dst_addr = s.solver.BVV(0x1000, 64)
    src = s.solver.BVS("src", 32)
    src_addr = s.solver.BVV(0x2000, 64)
    cmplen = s.solver.BVS("len", 64)

    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)
    r = memcmp(s, arguments=[dst_addr, src_addr, cmplen])

    # look at effects of different lengths
    s1 = s.copy()
    s1.add_constraints(cmplen == 1)
    s1.add_constraints(r == 0)
    l.debug("... simplifying")
    s1.solver._solver.simplify()
    l.debug("... solving")
    assert s1.solver.eval_upto(src[31:24], 2) == [0x41]
    assert not s1.solver.unique(src[31:16])
    l.debug("... solved")

    s2 = s.copy()
    s2.add_constraints(cmplen == 2)
    s2.add_constraints(r == 0)
    assert s2.solver.eval_upto(s2.memory.load(src_addr, 2), 2) == [0x4141]
    assert not s2.solver.unique(s2.memory.load(src_addr, 3))

    s2u = s.copy()
    s2u.add_constraints(cmplen == 2)
    s2u.add_constraints(r == 1)
    assert not s2u.solver.solution(s2u.memory.load(src_addr, 2), 0x4141)


def test_strncpy():
    l.info("concrete src, concrete dst, concrete len")
    l.debug("... full copy")
    s = SimState(arch="AMD64", mode="symbolic")
    dst = s.solver.BVV(0x41414100, 32)
    dst_addr = s.solver.BVV(0x1000, 64)
    src = s.solver.BVV(0x42420000, 32)
    src_addr = s.solver.BVV(0x2000, 64)

    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)
    strncpy(s, arguments=[dst_addr, src_addr, s.solver.BVV(3, 64)])
    new_dst = s.memory.load(dst_addr, 4, endness="Iend_BE")
    assert s.solver.eval(new_dst, cast_to=bytes) == b"BB\x00\x00"

    l.debug("... partial copy")
    s = SimState(arch="AMD64", mode="symbolic")
    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)
    strncpy(s, arguments=[dst_addr, src_addr, s.solver.BVV(2, 64)])
    new_dst = s.memory.load(dst_addr, 4, endness="Iend_BE")
    assert s.solver.eval_upto(new_dst, 2, cast_to=bytes) == [b"BBA\x00"]

    l.info("symbolic src, concrete dst, concrete len")
    s = SimState(arch="AMD64", mode="symbolic")
    dst = s.solver.BVV(0x41414100, 32)
    dst_addr = s.solver.BVV(0x1000, 64)
    src = s.solver.BVS("src", 32)
    src_addr = s.solver.BVV(0x2000, 64)

    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)

    # make sure it copies it all
    s.add_constraints(strlen(s, arguments=[src_addr]) == 2)

    # sanity check
    s_false = s.copy()
    s_false.add_constraints(strlen(s_false, arguments=[src_addr]) == 3)
    assert not s_false.satisfiable()

    strncpy(s, arguments=[dst_addr, src_addr, 3])
    assert s.satisfiable()
    c = strcmp(s, arguments=[dst_addr, src_addr])

    assert s.solver.eval_upto(c, 10) == [0]

    l.info("symbolic src, concrete dst, symbolic len")
    s = SimState(arch="AMD64", mode="symbolic")
    dst = s.solver.BVV(0x41414100, 32)
    dst_addr = s.solver.BVV(0x1000, 64)
    src = s.solver.BVS("src", 32)
    src_addr = s.solver.BVV(0x2000, 64)
    maxlen = s.solver.BVS("len", 64)

    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)

    # make sure it copies it all
    s.add_constraints(strlen(s, arguments=[src_addr]) == 2)
    strncpy(s, arguments=[dst_addr, src_addr, maxlen])
    c = strcmp(s, arguments=[dst_addr, src_addr])

    s_match = s.copy()
    s_match.add_constraints(c == 0)
    assert s_match.solver.min_int(maxlen) == 3

    s_nomatch = s.copy()
    s_nomatch.add_constraints(c != 0)
    assert s_nomatch.solver.max_int(maxlen) == 2

    l.info("concrete src, concrete dst, symbolic len")
    l.debug("... full copy")
    s = SimState(arch="AMD64", mode="symbolic")

    dst = s.solver.BVV(0x41414100, 32)
    dst_addr = s.solver.BVV(0x1000, 64)
    src = s.solver.BVV(0x42420000, 32)
    src_addr = s.solver.BVV(0x2000, 64)
    maxlen = s.solver.BVS("len", 64)

    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)
    strncpy(s, arguments=[dst_addr, src_addr, maxlen])
    r = s.memory.load(dst_addr, 4, endness="Iend_BE")
    # print repr(r.solver.eval_upto(r, 10, cast_to=bytes))
    assert sorted(s.solver.eval_upto(r, 10, cast_to=bytes)) == [b"AAA\x00", b"BAA\x00", b"BB\x00\x00", b"BBA\x00"]


def test_strcpy():
    l.info("concrete src, concrete dst")

    l.debug("... full copy")
    s = SimState(arch="AMD64", mode="symbolic")
    dst = s.solver.BVV(0x41414100, 32)
    dst_addr = s.solver.BVV(0x1000, 64)
    src = s.solver.BVV(0x42420000, 32)
    src_addr = s.solver.BVV(0x2000, 64)
    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)
    strcpy(s, arguments=[dst_addr, src_addr])
    new_dst = s.memory.load(dst_addr, 4, endness="Iend_BE")
    assert s.solver.eval(new_dst, cast_to=bytes) == b"BB\x00\x00"

    l.info("symbolic src, concrete dst")
    dst = s.solver.BVV(0x41414100, 32)
    dst_addr = s.solver.BVV(0x1000, 64)
    src = s.solver.BVS("src", 32)
    src_addr = s.solver.BVV(0x2000, 64)

    s = SimState(arch="AMD64", mode="symbolic")
    s.memory.store(dst_addr, dst)
    s.memory.store(src_addr, src)

    ln = strlen(s, arguments=[src_addr])

    strcpy(s, arguments=[dst_addr, src_addr])

    cm = strcmp(s, arguments=[dst_addr, src_addr])

    s.add_constraints(cm == 0)

    s.add_constraints(ln == 15)
    # readsize = 16
    # both_strs = s.solver.Concat(
    #     *[s.memory.load(dst_addr, readsize, endness="Iend_BE"), s.memory.load(src_addr, readsize, endness="Iend_BE")]
    # )
    # for i in s.solver.eval_upto(both_strs, 50, cast_to=bytes):
    #     assert s.solver.eval_upto(10) == [0]
    #     assert s.solver.solution(s.memory.load(dst_addr, 4, endness="Iend_BE"), 0x42434400)
    #     assert s.solver.solution(s.memory.load(dst_addr, 4, endness="Iend_BE"), 0x42434445)
    #     assert s.solver.solution(s.memory.load(dst_addr, 4, endness="Iend_BE"), 0x00414100)
    #     assert not s.solver.solution(s.memory.load(dst_addr, 4, endness="Iend_BE"), 0x00010203)


def broken_sprintf():
    l.info("concrete src, concrete dst, concrete len")
    s = SimState(mode="symbolic", arch="PPC32")
    format_str = s.solver.BVV(0x25640000, 32)
    format_addr = s.solver.BVV(0x2000, 32)
    # dst = s.solver.BVV("destination", 128)
    dst_addr = s.solver.BVV(0x1000, 32)
    arg = s.solver.BVS("some_number", 32)

    s.memory.store(format_addr, format_str)

    sprintf(s, arguments=[dst_addr, format_addr, arg])

    for i in range(9):
        j = random.randint(10**i, 10 ** (i + 1))
        s2 = s.copy()
        s2.add_constraints(arg == j)
        # print s2.solver.eval_upto(s2.memory.load(dst_addr, i+2), 2, cast_to=bytes), repr(b"%d\x00" % j)
        assert s2.solver.eval_upto(s2.memory.load(dst_addr, i + 2), 2, cast_to=bytes) == [b"%d\x00" % j]

    s2 = s.copy()
    s2.add_constraints(arg == 0)
    # print s2.solver.eval_upto(s2.memory.load(dst_addr, 2), 2, cast_to=bytes), repr(b"%d\x00" % 0)
    assert s2.solver.eval_upto(s2.memory.load(dst_addr, 2), 2, cast_to=bytes) == [b"%d\x00" % 0]


def test_memset():
    l.info("concrete src, concrete dst, concrete len")
    s = SimState(arch="PPC32", mode="symbolic")
    dst = s.solver.BVV(0, 128)
    dst_addr = s.solver.BVV(0x1000, 32)
    char = s.solver.BVV(0x00000041, 32)
    char2 = s.solver.BVV(0x50505050, 32)
    length = s.solver.BVS("some_length", 32)

    s.memory.store(dst_addr, dst)
    memset(s, arguments=[dst_addr, char, s.solver.BVV(3, 32)])
    assert s.solver.eval(s.memory.load(dst_addr, 4)) == 0x41414100

    l.debug("Symbolic length")
    s = SimState(arch="PPC32", mode="symbolic")
    s.memory.store(dst_addr, dst)
    length = s.solver.BVS("some_length", 32)
    s.add_constraints(length < 10)
    memset(s, arguments=[dst_addr, char2, length])

    l.debug("Trying 2")
    s_two = s.copy()
    s_two.add_constraints(length == 2)
    assert s_two.solver.eval(s_two.memory.load(dst_addr, 4)) == 0x50500000

    l.debug("Trying 0")
    s_zero = s.copy()
    s_zero.add_constraints(length == 0)
    assert s_zero.solver.eval(s_zero.memory.load(dst_addr, 4)) == 0x00000000

    l.debug("Trying 5")
    s_five = s.copy()
    s_five.add_constraints(length == 5)
    assert s_five.solver.eval(s_five.memory.load(dst_addr, 6)) == 0x505050505000


def test_strchr():
    l.info("concrete haystack and needle")
    s = SimState(arch="AMD64", mode="symbolic")
    str_haystack = s.solver.BVV(0x41424300, 32)
    str_needle = s.solver.BVV(0x42, 64)
    addr_haystack = s.solver.BVV(0x10, 64)
    s.memory.store(addr_haystack, str_haystack, endness="Iend_BE")

    ss_res = strchr(s, arguments=[addr_haystack, str_needle])
    assert s.solver.unique(ss_res)
    assert s.solver.eval(ss_res) == 0x11

    l.info("concrete haystack, symbolic needle")
    s = SimState(arch="AMD64", mode="symbolic")
    str_haystack = s.solver.BVV(0x41424300, 32)
    str_needle = s.solver.BVS("wtf", 64)
    chr_needle = str_needle[7:0]
    addr_haystack = s.solver.BVV(0x10, 64)
    s.memory.store(addr_haystack, str_haystack, endness="Iend_BE")

    ss_res = strchr(s, arguments=[addr_haystack, str_needle])
    assert not s.solver.unique(ss_res)
    assert len(s.solver.eval_upto(ss_res, 10)) == 5

    s_match = s.copy()
    s_nomatch = s.copy()
    s_match.add_constraints(ss_res != 0)
    s_nomatch.add_constraints(ss_res == 0)

    assert s_match.satisfiable()
    assert s_nomatch.satisfiable()
    assert len(s_match.solver.eval_upto(chr_needle, 300)) == 4
    assert len(s_nomatch.solver.eval_upto(chr_needle, 300)) == 252
    assert sorted(s_match.solver.eval_upto(ss_res, 300)) == [0x10, 0x11, 0x12, 0x13]
    assert sorted(s_match.solver.eval_upto(chr_needle, 300)) == [0x00, 0x41, 0x42, 0x43]

    s_match.memory.store(ss_res, s_match.solver.BVV(0x44, 8))
    assert sorted(s_match.solver.eval_upto(s_match.memory.load(0x10, 1), 300)) == [0x41, 0x44]
    assert sorted(s_match.solver.eval_upto(s_match.memory.load(0x11, 1), 300)) == [0x42, 0x44]
    assert sorted(s_match.solver.eval_upto(s_match.memory.load(0x12, 1), 300)) == [0x43, 0x44]
    assert sorted(s_match.solver.eval_upto(s_match.memory.load(0x13, 1), 300)) == [0x00, 0x44]

    # l.info("symbolic haystack, symbolic needle")
    # s = SimState(arch="AMD64", mode="symbolic")
    # s.libc.buf_symbolic_bytes = 5
    # addr_haystack = s.solver.BVV(0x10, 64)
    # addr_needle = s.solver.BVV(0xb0, 64)
    # len_needle = strlen(s, inline=True, arguments=[addr_needle])

    # ss_res = strstr(s, inline=True, arguments=[addr_haystack, addr_needle])
    # ss_val = s.expr_value(ss_res)

    # assert not ss_val.is_unique()
    # assert len(ss_val.solver.eval_upto(100)) == s.libc.buf_symbolic_bytes

    # s_match = s.copy()
    # s_nomatch = s.copy()
    # s_match.add_constraints(ss_res != 0)
    # s_nomatch.add_constraints(ss_res == 0)

    # match_cmp = strncmp(s_match, inline=True, arguments=[ss_res, addr_needle, len_needle])
    # match_cmp_val = s_match.expr_value(match_cmp)
    # assert match_cmp_val.solver.eval_upto(10) == [0]

    # r_mm = strstr(s_match, inline=True, arguments=[addr_haystack, addr_needle])
    # s_match.add_constraints(r_mm == 0)
    # assert not s_match.satisfiable()

    # assert s_nomatch.satisfiable()
    # s_nss = s_nomatch.copy()
    # nomatch_ss = strstr(s_nss, inline=True, arguments=[addr_haystack, addr_needle])
    # s_nss.add_constraints(nomatch_ss != 0)
    # assert not s_nss.satisfiable()


def broken_strtok_r():
    l.debug("CONCRETE MODE")
    s = SimState(arch="AMD64", mode="symbolic")
    s.memory.store(100, s.solver.BVV(0x4141414241414241424300, 88), endness="Iend_BE")
    s.memory.store(200, s.solver.BVV(0x4200, 16), endness="Iend_BE")
    str_ptr = s.solver.BVV(100, s.arch.bits)
    delim_ptr = s.solver.BVV(200, s.arch.bits)
    state_ptr = s.solver.BVV(300, s.arch.bits)

    st1 = strtok_r(s, arguments=[str_ptr, delim_ptr, state_ptr])
    assert s.solver.eval_upto(st1, 10) == [104]
    assert s.solver.eval_upto(s.memory.load(st1 - 1, 1), 10) == [0]
    assert s.solver.eval_upto(s.memory.load(200, 2), 10) == [0x4200]

    st2 = strtok_r(s, arguments=[s.solver.BVV(0, s.arch.bits), delim_ptr, state_ptr])
    assert s.solver.eval_upto(st2, 10) == [107]
    assert s.solver.eval_upto(s.memory.load(st2 - 1, 1), 10) == [0]

    st3 = strtok_r(s, arguments=[s.solver.BVV(0, s.arch.bits), delim_ptr, state_ptr])
    assert s.solver.eval_upto(st3, 10) == [109]
    assert s.solver.eval_upto(s.memory.load(st3 - 1, 1), 10) == [0]

    st4 = strtok_r(s, arguments=[s.solver.BVV(0, s.arch.bits), delim_ptr, state_ptr])
    assert s.solver.eval_upto(st4, 10) == [0]
    assert s.solver.eval_upto(s.memory.load(300, s.arch.bytes, endness=s.arch.memory_endness), 10) == [109]

    st5 = strtok_r(s, arguments=[s.solver.BVV(0, s.arch.bits), delim_ptr, state_ptr])
    assert s.solver.eval_upto(st5, 10) == [0]
    assert s.solver.eval_upto(s.memory.load(300, s.arch.bytes, endness=s.arch.memory_endness), 10) == [109]

    s.memory.store(1000, s.solver.BVV(0x4141414241414241424300, 88), endness="Iend_BE")
    s.memory.store(2000, s.solver.BVV(0x4200, 16), endness="Iend_BE")
    str_ptr = s.solver.BVV(1000, s.arch.bits)
    delim_ptr = s.solver.BVV(2000, s.arch.bits)
    state_ptr = s.solver.BVV(3000, s.arch.bits)

    st1 = strtok_r(s, arguments=[str_ptr, delim_ptr, state_ptr])
    assert s.solver.eval_upto(st1, 10) == [1004]
    assert s.solver.eval_upto(s.memory.load(st1 - 1, 1), 10) == [0]
    assert s.solver.eval_upto(s.memory.load(2000, 2), 10) == [0x4200]

    st2 = strtok_r(s, arguments=[s.solver.BVV(0, s.arch.bits), delim_ptr, state_ptr])
    assert s.solver.eval_upto(st2, 10) == [1007]
    assert s.solver.eval_upto(s.memory.load(st2 - 1, 1), 10) == [0]

    st3 = strtok_r(s, arguments=[s.solver.BVV(0, s.arch.bits), delim_ptr, state_ptr])
    assert s.solver.eval_upto(st3, 10) == [1009]
    assert s.solver.eval_upto(s.memory.load(st3 - 1, 1), 10) == [0]

    st4 = strtok_r(s, arguments=[s.solver.BVV(0, s.arch.bits), delim_ptr, state_ptr])
    assert s.solver.eval_upto(st4, 10) == [0]
    assert s.solver.eval_upto(s.memory.load(3000, s.arch.bytes, endness=s.arch.memory_endness), 10) == [1009]

    st5 = strtok_r(s, arguments=[s.solver.BVV(0, s.arch.bits), delim_ptr, state_ptr])
    assert s.solver.eval_upto(st5, 10) == [0]
    assert s.solver.eval_upto(s.memory.load(3000, s.arch.bytes, endness=s.arch.memory_endness), 10) == [1009]

    s = SimState(arch="AMD64", mode="symbolic")
    str_ptr = s.solver.BVV(100, s.arch.bits)
    delim_ptr = s.solver.BVV(200, s.arch.bits)
    state_ptr = s.solver.BVV(300, s.arch.bits)

    s.add_constraints(s.memory.load(delim_ptr, 1) != 0)

    st1 = strtok_r(s, arguments=[str_ptr, delim_ptr, state_ptr])
    s.add_constraints(st1 != 0)
    assert s.solver.eval_upto(s.memory.load(st1 - 1, 1), 10) == [0]


def test_getc():
    s = make_state_with_stdin(b"1234")
    stdin = s.posix.get_fd(0)
    s.mem[0x1000 + 0x70].int = 0

    assert s.solver.eval_one(stdin.tell()) == 0

    # The argument of getc should be a FILE *
    c = getc(s, [0x1000])
    assert s.solver.eval_upto(c, 300) == [0x31]
    assert s.solver.eval_upto(stdin.tell(), 300) == [1]

    c = getc(s, [0x1000])
    assert s.solver.eval_upto(c, 300) == [0x32]
    assert s.solver.eval_upto(stdin.tell(), 300) == [2]

    c = getc(s, [0x1000])
    assert s.solver.eval_upto(c, 300) == [0x33]
    assert s.solver.eval_upto(stdin.tell(), 300) == [3]

    c = getc(s, [0x1000])
    assert s.solver.eval_upto(c, 300) == [0x34]
    assert s.solver.eval_upto(stdin.tell(), 300) == [4]


def test_getchar():
    s = make_state_with_stdin(b"1234")
    stdin = s.posix.get_fd(0)

    assert s.solver.eval_upto(stdin.tell(), 300) == [0]
    c = getchar(s, arguments=[])
    assert s.solver.eval_upto(c, 300) == [0x31]
    assert s.solver.eval_upto(stdin.tell(), 300) == [1]

    c = getchar(s, arguments=[])
    assert s.solver.eval_upto(c, 300) == [0x32]
    assert s.solver.eval_upto(stdin.tell(), 300) == [2]

    c = getchar(s, arguments=[])
    assert s.solver.eval_upto(c, 300) == [0x33]
    assert s.solver.eval_upto(stdin.tell(), 300) == [3]

    c = getchar(s, arguments=[])
    assert s.solver.eval_upto(c, 300) == [0x34]
    assert s.solver.eval_upto(stdin.tell(), 300) == [4]


def test_scanf():
    s = make_state_with_stdin(b"Hello\n")
    s.memory.store(0x2000, b"%1s\0")
    scanf(s, [0x2000, 0x1000])
    assert s.solver.eval_upto(s.memory.load(0x1000, 2), 2, cast_to=bytes) == [b"H\x00"]


def test_strcmp():
    l.info("concrete a, concrete b")
    s = SimState(arch="AMD64", mode="symbolic")
    a_addr = s.solver.BVV(0x10, 64)
    b_addr = s.solver.BVV(0xB0, 64)

    s.memory.store(a_addr, b"heck\x00")
    s.memory.store(b_addr, b"heck\x00")

    r = strcmp(s, arguments=[a_addr, b_addr])
    assert s.solver.eval_upto(r, 2) == [0]

    l.info("concrete a, empty b")
    s = SimState(arch="AMD64", mode="symbolic")
    a_addr = s.solver.BVV(0x10, 64)
    b_addr = s.solver.BVV(0xB0, 64)

    s.memory.store(a_addr, b"heck\x00")
    s.memory.store(b_addr, b"\x00")

    r = strcmp(s, arguments=[a_addr, b_addr])
    assert s.solver.eval_upto(r, 2) == [1]

    l.info("empty a, concrete b")
    s = SimState(arch="AMD64", mode="symbolic")
    a_addr = s.solver.BVV(0x10, 64)
    b_addr = s.solver.BVV(0xB0, 64)

    s.memory.store(a_addr, b"\x00")
    s.memory.store(b_addr, b"heck\x00")

    r = strcmp(s, arguments=[a_addr, b_addr])
    assert s.solver.eval_upto(r, 2) == [0xFFFFFFFF]

    l.info("empty a, empty b")
    s = SimState(arch="AMD64", mode="symbolic")
    a_addr = s.solver.BVV(0x10, 64)
    b_addr = s.solver.BVV(0xB0, 64)

    s.memory.store(a_addr, b"\x00")
    s.memory.store(b_addr, b"\x00")

    r = strcmp(s, arguments=[a_addr, b_addr])
    assert s.solver.eval_upto(r, 2) == [0]


def test_wcscmp():
    # concrete cases for the wide char version sufficiently overlap with strcmp and friends
    l.info("concrete a, symbolic b")
    s = SimState(arch="AMD64", mode="symbolic")
    heck = "heck\x00".encode("utf-16")[2:]  # remove encoding prefix
    a_addr = s.solver.BVV(0x10, 64)
    b_addr = s.solver.BVV(0xB0, 64)
    b_bvs = s.solver.BVS("b", len(heck) * 8)

    s.memory.store(a_addr, heck)
    s.memory.store(b_addr, b_bvs)

    r = wcscmp(s, arguments=[a_addr, b_addr])

    solutions = s.solver.eval_upto(b_bvs, 2, cast_to=bytes, extra_constraints=(r == 0,))
    assert solutions == [heck]


def test_string_without_null():
    s = SimState(arch="AMD64", mode="symbolic")
    str_ = b"abcd"
    str_addr = s.solver.BVV(0x10, 64)
    s.memory.store(str_addr, str_)
    assert s.solver.eval(s.mem[str_addr].string.resolved, cast_to=bytes) == b"abcd"


def run_all():
    def print_test_name(name):
        print("#" * (len(name) + 8))
        print("###", name, "###")
        print("#" * (len(name) + 8))

    functions = globals()
    all_functions = dict(filter((lambda kv: kv[0].startswith("test_")), functions.items()))
    for f in sorted(all_functions.keys()):
        if hasattr(all_functions[f], "__call__"):
            print_test_name(f)
            all_functions[f]()


if __name__ == "__main__":
    if len(sys.argv) > 1:
        globals()["test_" + sys.argv[1]]()
    else:
        run_all()



import os

import angr

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..")


def _ultra_oppologist(p, s):
    old_ops = dict(angr.engines.vex.claripy.irop.operations)
    try:
        angr.engines.vex.claripy.irop.operations.clear()
        angr.engines.vex.claripy.irop.operations["Iop_Add32"] = old_ops["Iop_Add32"]

        pg = p.factory.simulation_manager(s)
        pg.use_technique(angr.exploration_techniques.Oppologist())
        pg.explore()

        return pg
    finally:
        angr.engines.vex.claripy.irop.operations.update(old_ops)


def test_fauxware_oppologist():
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "i386", "fauxware"))
    s = p.factory.full_init_state(remove_options={angr.options.LAZY_SOLVES, angr.options.EXTENDED_IROP_SUPPORT})

    pg = _ultra_oppologist(p, s)
    assert len(pg.deadended) == 1
    assert len(pg.deadended[0].posix.dumps(0)) == 18
    stdout = pg.deadended[0].posix.dumps(1)
    if b"trusted user" in stdout:
        assert stdout.count(b"\n") == 3
    else:
        assert stdout.count(b"\n") == 2


def test_cromu_70():
    p = angr.Project(os.path.join(test_location, "binaries", "tests", "cgc", "CROMU_00070"))
    inp = bytes.fromhex(
        "030e000001000001001200010000586d616ce000000600030000040dd0000000000600000606000006030e000001000001003200010000586d616ce0030000000000030e000001000001003200010000586d616ce003000000000006000006030e000001000001003200010000586d616ce0030000df020000"
    )
    s = p.factory.full_init_state(
        add_options={angr.options.UNICORN},
        remove_options={angr.options.LAZY_SOLVES, angr.options.SUPPORT_FLOATING_POINT},
        stdin=inp,
    )

    # import traceit
    pg = p.factory.simulation_manager(s)
    pg.use_technique(angr.exploration_techniques.Oppologist())
    pg.run(n=50)
    assert pg.one_active.history.block_count > 1500


def run_all():
    functions = globals()
    all_functions = dict(filter((lambda kv: kv[0].startswith("test_")), functions.items()))
    for f in sorted(all_functions.keys()):
        if hasattr(all_functions[f], "__call__"):
            print(f)
            all_functions[f]()


if __name__ == "__main__":
    run_all()



import angr
import os

import logging

l = logging.getLogger("angr.tests.test_multi_open_file")

test_location = os.path.dirname(os.path.realpath(__file__))


def test_multi_open_file():
    test_bin = os.path.join(test_location, "..", "..", "binaries", "tests", "x86_64", "test_multi_open_file")
    # auto_load_libs cannot be disabled as the test fails
    b = angr.Project(test_bin)

    pg = b.factory.simulation_manager()
    pg.active[0].options.discard("LAZY_SOLVES")
    pg.explore()

    assert len(pg.deadended) == 1

    # See the source file in binaries/tests_src/test_multi_open_file.c
    # for the tests run
    for p in pg.deadended:
        assert p.posix.dumps(2) == b""

        # Check that the temp file was deleted
        assert p.fs._files == {}

        # Check that the deleted temp file contained the appropriate string
        for event in p.history.events:
            if event.type == "fs_unlink":
                simfile = p.fs.unlinks[event.objects["unlink_idx"]][1]
                assert simfile.concretize() == b"foobar and baz"
                break
        else:
            assert False


if __name__ == "__main__":
    test_multi_open_file()



import logging
import os

import angr

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")


def run_tracker(track_mem, use_bp):
    p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)
    p.analyses.CFGFast()
    main = p.kb.functions["main"]
    sp = p.arch.sp_offset
    regs = {sp}
    if use_bp:
        bp = p.arch.bp_offset
        regs.add(bp)
    sptracker = p.analyses.StackPointerTracker(main, regs, track_memory=track_mem)
    sp_result = sptracker.offset_after(0x4007D4, sp)
    if use_bp:
        bp_result = sptracker.offset_after(0x4007D4, bp)
        return sp_result, bp_result
    else:
        return sp_result


def init_tracker(track_mem):
    p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)
    p.analyses.CFGFast()
    main = p.kb.functions["main"]
    sp = p.arch.sp_offset
    regs = {sp}
    sptracker = p.analyses.StackPointerTracker(main, regs, track_memory=track_mem)
    return sptracker, sp


def test_stack_pointer_tracker():
    sp_result, bp_result = run_tracker(track_mem=True, use_bp=True)
    assert sp_result == 8
    assert bp_result == 0


def test_stack_pointer_tracker_no_mem():
    sp_result, bp_result = run_tracker(track_mem=False, use_bp=True)
    assert sp_result == 8
    assert bp_result is None


def test_stack_pointer_tracker_just_sp():
    sp_result = run_tracker(track_mem=False, use_bp=False)
    assert sp_result is None


def test_stack_pointer_tracker_offset_block():
    sptracker, sp = init_tracker(track_mem=False)
    sp_result = sptracker.offset_after_block(0x40071D, sp)
    assert sp_result is not None
    sp_result = sptracker.offset_after_block(0x400700, sp)
    assert sp_result is None
    sp_result = sptracker.offset_before_block(0x40071D, sp)
    assert sp_result is not None
    sp_result = sptracker.offset_before_block(0x400700, sp)
    assert sp_result is None


if __name__ == "__main__":
    logging.getLogger("angr.analyses.stack_pointer_tracker").setLevel(logging.INFO)
    test_stack_pointer_tracker()
    test_stack_pointer_tracker_no_mem()
    test_stack_pointer_tracker_just_sp()
    test_stack_pointer_tracker_offset_block()



#!/usr/bin/env python

import logging

l = logging.getLogger("angr.tests.slicing")

import time
import angr
from angr.utils.constants import DEFAULT_STATEMENT

# Load the tests
import os

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")


def test_find_exits():
    slicing_test = angr.Project(
        os.path.join(test_location, "x86_64", "cfg_1"),
        use_sim_procedures=True,
        default_analysis_mode="symbolic",
        auto_load_libs=False,
    )

    l.info("Unit test for BackwardSlice._find_exits()")
    cfg = slicing_test.analyses.CFGEmulated(context_sensitivity_level=2, keep_state=True)
    cdg = slicing_test.analyses.CDG(cfg)
    ddg = slicing_test.analyses.DDG(cfg)

    source = cfg.get_any_node(0x40059E)

    # Test the conditional exit
    target = cfg.get_any_node(0x400594)
    bs_1 = slicing_test.analyses.BackwardSlice(cfg, cdg, ddg, targets=[(target, -1)], no_construct=True)
    all_exits = bs_1._find_exits(source, target)

    assert all_exits == {18: [0x400594], DEFAULT_STATEMENT: None}

    # Test the default exit
    target = cfg.get_any_node(0x4005A4)
    bs_2 = slicing_test.analyses.BackwardSlice(cfg, cdg, ddg, targets=[(target, -1)], no_construct=True)
    all_exits = bs_2._find_exits(source, target)

    assert all_exits == {18: [0x400594], DEFAULT_STATEMENT: [0x4005A4]}


def test_control_flow_slicing():
    slicing_test = angr.Project(
        os.path.join(test_location, "x86_64", "cfg_1"),
        use_sim_procedures=True,
        default_analysis_mode="symbolic",
        auto_load_libs=False,
    )
    l.info("Control Flow Slicing")
    start = time.time()
    cfg = slicing_test.analyses.CFGEmulated(context_sensitivity_level=2)
    end = time.time()
    duration = end - start
    l.info("CFG generation is done in %f seconds.", duration)

    target = cfg.get_any_node(0x400594)
    bs = slicing_test.analyses.BackwardSlice(cfg, None, None, targets=[(target, -1)], control_flow_slice=True)
    anno_cfg = bs.annotated_cfg()
    assert anno_cfg.get_whitelisted_statements(0x40057C) is None
    assert anno_cfg.get_whitelisted_statements(0x400594) is None
    assert anno_cfg.get_whitelisted_statements(0x4005A4) == []


def broken_backward_slice():
    # TODO: Fix this test case

    slicing_test = angr.Project(
        os.path.join(test_location, "x86_64", "cfg_1"),
        use_sim_procedures=True,
        default_analysis_mode="symbolic",
        auto_load_libs=False,
    )

    l.info("Control Flow Slicing")

    cfg = slicing_test.analyses.CFGEmulated(
        context_sensitivity_level=2, keep_state=True, state_add_options=angr.sim_options.refs
    )
    cdg = slicing_test.analyses.CDG(cfg=cfg)
    ddg = slicing_test.analyses.DDG(cfg=cfg)

    target = cfg.get_any_node(0x4005D3)
    bs = slicing_test.analyses.BackwardSlice(cfg, cdg, ddg, targets=[(target, -1)], control_flow_slice=False)
    anno_cfg = bs.annotated_cfg()
    assert anno_cfg.get_whitelisted_statements(0x40057C) == [2, 3, 7, 20, 21]
    assert anno_cfg.get_whitelisted_statements(0x400594) == [1, 17, 18, 19, 20]
    assert anno_cfg.get_whitelisted_statements(0x4005A4) == []
    assert anno_cfg.get_whitelisted_statements(0x4005CD) == [1, 2, 3, 5, 6, 11, 12, 13, 14, 15, 16, 17, 18, 19]


def test_last_branching_statement():
    slicing_test = angr.Project(
        os.path.join(test_location, "armel", "fauxware"),
        use_sim_procedures=True,
        auto_load_libs=False,
    )
    l.info("Testing _search_for_last_branching_statement.")

    # The IRSB:

    # ------ IMark(0x86dc, 4, 0) ------
    # t1 = GET:I32(r0)
    # PUT(pc) = 0x000086e0
    # ------ IMark(0x86e0, 4, 0) ------
    # t14 = GET:I32(r11)
    # t13 = Sub32(t14,0x00000024)
    # STle(t13) = t1
    # PUT(pc) = 0x000086e4
    # ------ IMark(0x86e4, 4, 0) ------
    # t15 = t13
    # t8 = LDle:I32(t15)
    # PUT(r3) = t8
    # ------ IMark(0x86e8, 4, 0) ------
    # PUT(cc_op) = 0x00000002
    # PUT(cc_dep1) = t8
    # PUT(cc_dep2) = 0x00000000
    # PUT(cc_ndep) = 0x00000000
    # PUT(pc) = 0x000086ec
    # ------ IMark(0x86ec, 4, 0) ------
    # t26 = CmpEQ32(t8,0x00000000)
    # t25 = 1Uto32(t26)
    # t27 = 32to1(t25)
    # if (t27) { PUT(68) = 0x86f8; Ijk_Boring }

    target_state = slicing_test.factory.blank_state(addr=0x86DC)
    simgr = slicing_test.factory.simgr(target_state)
    simgr.step()
    target = simgr.active[0]
    l.debug("IRSB:")
    for line in target.scratch.irsb._pp_str().split("\n"):
        l.debug(line)

    bs = slicing_test.analyses.BackwardSlice(None, None, None, targets=[(target, -1)], no_construct=True)

    stmt_idx, tmp = bs._last_branching_statement(target.scratch.irsb.statements)

    assert stmt_idx == 22
    assert tmp == 27


def test_fauxware():
    b = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)
    cfg = b.analyses.CFGEmulated(keep_state=True, state_add_options=angr.sim_options.refs, context_sensitivity_level=2)
    cdg = b.analyses.CDG(cfg)
    ddg = b.analyses.DDG(cfg)
    target_func = cfg.kb.functions.function(name="exit")
    target_node = cfg.get_any_node(target_func.addr)

    bs = b.analyses.BackwardSlice(cfg, cdg=cdg, ddg=ddg, targets=[(target_node, -1)])

    # Make sure dbg_repr() works
    l.debug(bs.dbg_repr())


if __name__ == "__main__":
    test_fauxware()
    test_find_exits()
    test_last_branching_statement()
    test_control_flow_slicing()
    # test_backward_slice()



import tempfile
import pickle
import shutil
import angr
import os

internaltest_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")
internaltest_files = [
    "argc_decide",
    "argc_symbol",
    "argv_test",
    "counter",
    "fauxware",
    "fauxware.idb",
    "manysum",
    "pw",
    "strlen",
    "test_arrays",
    "test_division",
    "test_loops",
]
internaltest_arch = ["i386", "armel"]


def internaltest_vfg(p, cfg):
    vfg = p.analyses.VFG(cfg=cfg)
    v = angr.vaults.VaultDict()
    state = v.dumps(vfg)
    vfg2 = v.loads(state)
    assert vfg.final_states == vfg2.final_states
    assert set(vfg.graph.nodes()) == set(vfg2.graph.nodes())


def internaltest_cfg(p):
    state = tempfile.TemporaryFile()

    cfg = p.analyses.CFGEmulated()
    pickle.dump(cfg, state, -1)

    state.seek(0)
    cfg2 = pickle.load(state)
    assert set(cfg.model.nodes()) == set(cfg2.model.nodes())
    assert cfg.unresolvables == cfg2.unresolvables
    assert set(cfg.deadends) == set(cfg2.deadends)

    return cfg


def internaltest_cfgfast(p):
    state = tempfile.TemporaryFile()

    cfg = p.analyses.CFGFast()

    # generate capstone blocks
    main_function = cfg.functions.function(name="main")
    for b in main_function.blocks:
        b.capstone

    pickle.dump(cfg, state, -1)

    state.seek(0)
    cfg2 = pickle.load(state)
    assert set(cfg.model.nodes()) == set(cfg2.model.nodes())


def internaltest_project(fpath):
    tpath = tempfile.mktemp()
    shutil.copy(fpath, tpath)

    p = angr.Project(tpath, auto_load_libs=False)
    state = pickle.dumps(p, -1)
    loaded_p = pickle.loads(state)
    assert p is not loaded_p
    assert p.arch == loaded_p.arch
    assert p.filename == loaded_p.filename
    assert p.entry == loaded_p.entry

    simgr = loaded_p.factory.simulation_manager()
    simgr.run(n=10)
    assert len(simgr.errored) == 0


def test_analyses():
    p = angr.Project(os.path.join(internaltest_location, "i386", "fauxware"), load_options={"auto_load_libs": False})
    cfg = p.analyses.CFG()
    cfb = p.analyses.CFB(kb=cfg.kb)
    vrf = p.analyses.VariableRecoveryFast(p.kb.functions["main"])

    assert len(p.kb.functions) > 0
    assert len(pickle.loads(pickle.dumps(p.kb, -1)).functions) > 0

    state = pickle.dumps((p, cfg, cfb, vrf), -1)
    del p
    del cfg
    del cfb
    del vrf
    import gc

    gc.collect()

    p, cfg, cfb, vrf = pickle.loads(state)
    assert p.kb is not None
    assert p.kb.functions is not None
    assert cfg.kb is not None
    assert len(p.kb.functions) > 0


def test_serialization():
    for d in internaltest_arch:
        for f in internaltest_files:
            fpath = os.path.join(internaltest_location, d, f)
            if os.path.isfile(fpath) and os.access(fpath, os.X_OK):
                internaltest_project(fpath)

    p = angr.Project(os.path.join(internaltest_location, "i386", "fauxware"), load_options={"auto_load_libs": False})
    internaltest_cfgfast(p)

    cfg = internaltest_cfg(p)
    internaltest_vfg(p, cfg)


if __name__ == "__main__":
    test_analyses()
    test_serialization()



from typing import List

import ailment
from ailment.expression import Op

from ..structuring.structurer_nodes import ConditionNode
from ..utils import structured_node_is_simple_return
from .optimization_pass import SequenceOptimizationPass, OptimizationPassStage


class FlipBooleanCmp(SequenceOptimizationPass):
    """
    In the scenario in which a false node has no apparent successors, flip the condition on that if-stmt.
    This is only useful when StructuredCodeGenerator has simplify_else_scopes enabled, as this will allow the
    flipped if-stmt to remove the redundant else.
    """

    ARCHES = None
    PLATFORMS = None
    STAGE = OptimizationPassStage.AFTER_STRUCTURING
    NAME = "Flip small ret booleans"
    DESCRIPTION = "When false node has no successors, flip condition so else scope can be simplified later"

    def __init__(self, func, **kwargs):
        super().__init__(func, **kwargs)
        self._graph = kwargs.get("graph", None)
        self.analyze()

    def _check(self):
        condition_nodes = [node for node in self.seq.nodes if isinstance(node, ConditionNode) and node.false_node]
        return len(condition_nodes) > 0, condition_nodes

    def _analyze(self, cache=None):
        condition_nodes: List[ConditionNode] = cache or []
        for node in condition_nodes:
            if isinstance(node.condition, Op) and structured_node_is_simple_return(node.false_node, self._graph):
                node.condition = ailment.expression.negate(node.condition)
                node.true_node, node.false_node = node.false_node, node.true_node



import angr

import logging

l = logging.getLogger("angr_tests")

import os

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")


def test_nx():
    nx_amd64 = angr.Project(os.path.join(test_location, "x86_64", "memmove"), auto_load_libs=False)
    es = nx_amd64.factory.entry_state()

    # .text should be PROT_READ|PROT_EXEC
    assert es.solver.eval(es.memory.permissions(nx_amd64.entry)) == 5

    # load stack to initialize page
    es.memory.load(es.regs.sp, 4)

    # stack should be PROT_READ|PROT_WRITE
    assert es.solver.eval(es.memory.permissions(es.regs.sp)) == 3


if __name__ == "__main__":
    test_nx()



from typing import Optional, Tuple, Union, List, DefaultDict, TYPE_CHECKING
from collections import defaultdict
import logging

import networkx

from ailment import Block, AILBlockWalkerBase
from ailment.statement import ConditionalJump
from ailment.expression import Expression, BinaryOp, Const, Load

from angr.utils.graph import GraphUtils
from ..utils import first_nonlabel_statement, remove_last_statement
from ..structuring.structurer_nodes import IncompleteSwitchCaseHeadStatement, SequenceNode, MultiNode
from .optimization_pass import OptimizationPass, OptimizationPassStage, MultipleBlocksException

if TYPE_CHECKING:
    from ailment.expression import UnaryOp, Convert

_l = logging.getLogger(name=__name__)


class Case:
    """
    Describes a case in a switch-case construct.
    """

    __slots__ = (
        "original_node",
        "node_type",
        "variable_hash",
        "expr",
        "value",
        "target",
        "next_addr",
    )

    def __init__(
        self, original_node, node_type: Optional[str], variable_hash, expr, value: Union[int, str], target, next_addr
    ):
        self.original_node = original_node
        self.node_type = node_type
        self.variable_hash = variable_hash
        self.expr = expr
        self.value = value
        self.target = target
        self.next_addr = next_addr

    def __repr__(self):
        if self.value == "default":
            return f"Case default@{self.target:#x}"
        return f"Case {repr(self.original_node)}@{self.target:#x}: {self.expr} == {self.value}"

    def __eq__(self, other):
        if not isinstance(other, Case):
            return False
        return (
            self.original_node == other.original_node
            and self.node_type == other.node_type
            and self.variable_hash == other.variable_hash
            and self.value == other.value
            and self.target == other.target
            and self.next_addr == other.next_addr
        )

    def __hash__(self):
        return hash(
            (Case, self.original_node, self.node_type, self.variable_hash, self.value, self.target, self.next_addr)
        )


class StableVarExprHasher(AILBlockWalkerBase):
    """
    Obtain a stable hash of an AIL expression with respect to all variables and all operations applied on variables.
    """

    def __init__(self, expr: Expression):
        super().__init__()
        self.expr = expr
        self._hash_lst = []

        self.walk_expression(expr)
        self.hash = hash(tuple(self._hash_lst))

    def _handle_expr(self, expr_idx: int, expr: Expression, stmt_idx: int, stmt, block: Optional[Block]):
        if hasattr(expr, "variable"):
            self._hash_lst.append(expr.variable)
        else:
            super()._handle_expr(expr_idx, expr, stmt_idx, stmt, block)

    def _handle_Load(self, expr_idx: int, expr: Load, stmt_idx: int, stmt, block: Optional[Block]):
        self._hash_lst.append("Load")
        super()._handle_expr(expr_idx, expr, stmt_idx, stmt, block)

    def _handle_BinaryOp(self, expr_idx: int, expr: BinaryOp, stmt_idx: int, stmt, block: Optional[Block]):
        self._hash_lst.append(expr.op)
        super()._handle_BinaryOp(expr_idx, expr, stmt_idx, stmt, block)

    def _handle_UnaryOp(self, expr_idx: int, expr: "UnaryOp", stmt_idx: int, stmt, block: Optional[Block]):
        self._hash_lst.append(expr.op)
        super()._handle_UnaryOp(expr_idx, expr, stmt_idx, stmt, block)

    def _handle_Const(self, expr_idx: int, expr: Const, stmt_idx: int, stmt, block: Optional[Block]):
        self._hash_lst.append((expr.value, expr.bits))

    def _handle_Convert(self, expr_idx: int, expr: "Convert", stmt_idx: int, stmt, block: Optional[Block]):
        self._hash_lst.append(expr.to_bits)
        super()._handle_Convert(expr_idx, expr, stmt_idx, stmt, block)


class LoweredSwitchSimplifier(OptimizationPass):
    """
    Recognize and simplify lowered switch-case constructs.
    """

    ARCHES = [
        "AMD64",
    ]
    PLATFORMS = ["linux", "windows"]
    STAGE = OptimizationPassStage.BEFORE_REGION_IDENTIFICATION
    NAME = "Convert lowered switch-cases (if-else) to switch-cases"
    DESCRIPTION = (
        "Convert lowered switch-cases (if-else) to switch-cases. Only works when the Phoenix structuring "
        "algorithm is in use."
    )
    STRUCTURING = ["phoenix"]

    def __init__(self, func, blocks_by_addr=None, blocks_by_addr_and_idx=None, graph=None, **kwargs):
        super().__init__(
            func, blocks_by_addr=blocks_by_addr, blocks_by_addr_and_idx=blocks_by_addr_and_idx, graph=graph, **kwargs
        )
        self.analyze()

    def _check(self):
        # TODO: More filtering
        return True, None

    def _analyze(self, cache=None):
        variablehash_to_cases = self._find_cascading_switch_variable_comparisons()

        if not variablehash_to_cases:
            return

        graph_copy = networkx.DiGraph(self._graph)
        self.out_graph = graph_copy
        node_to_heads = defaultdict(set)

        for _, caselists in variablehash_to_cases.items():
            for cases in caselists:
                original_nodes = [case.original_node for case in cases if case.value != "default"]
                original_head: Block = original_nodes[0]
                original_nodes = original_nodes[1:]

                case_addrs = {(case.original_node, case.value, case.target, case.next_addr) for case in cases}

                expr = cases[0].expr

                # create a fake switch-case head node
                switch_stmt = IncompleteSwitchCaseHeadStatement(
                    original_head.statements[-1].idx, expr, case_addrs, ins_addr=original_head.statements[-1].ins_addr
                )
                new_head = original_head.copy()
                # replace the last instruction of the head node with switch_node
                new_head.statements[-1] = switch_stmt
                # update the block
                self._update_block(original_head, new_head)

                # sanity check that no switch head points to either itself
                # or to any if-head that was merged into the new switch head; this
                # would result in a successor node no longer being present in the graph
                if any(onode not in graph_copy for onode in original_nodes):
                    self.out_graph = None
                    return

                # add edges between the head and case nodes
                for onode in original_nodes:
                    successors = list(graph_copy.successors(onode))
                    for succ in successors:
                        if succ not in original_nodes:
                            graph_copy.add_edge(new_head, succ)
                            node_to_heads[succ].add(new_head)
                    graph_copy.remove_node(onode)

        # find shared case nodes and make copies of them
        # note that this only solves cases where *one* node is shared between switch-cases. a more general solution
        # requires jump threading reverter.
        for succ_node, heads in node_to_heads.items():
            if len(heads) > 1:
                # each head gets a copy of the node!
                node_successors = list(graph_copy.successors(succ_node))
                next_id = 0 if succ_node.idx is None else succ_node.idx + 1
                graph_copy.remove_node(succ_node)
                for head in heads:
                    node_copy = succ_node.copy()
                    node_copy.idx = next_id
                    next_id += 1
                    graph_copy.add_edge(head, node_copy)
                    for succ in node_successors:
                        if succ is succ_node:
                            graph_copy.add_edge(node_copy, node_copy)
                        else:
                            graph_copy.add_edge(node_copy, succ)

    def _find_cascading_switch_variable_comparisons(self):
        sorted_nodes = GraphUtils.quasi_topological_sort_nodes(self._graph)
        variable_comparisons = {}
        for node in sorted_nodes:
            r = self._find_switch_variable_comparison_type_a(node)
            if r is not None:
                variable_comparisons[node] = ("a",) + r
                continue
            r = self._find_switch_variable_comparison_type_b(node)
            if r is not None:
                variable_comparisons[node] = ("b",) + r
                continue

        varhash_to_caselists: DefaultDict[int, List[List[Case]]] = defaultdict(list)

        for head in variable_comparisons:
            cases = []
            last_comp = None
            comp = head
            while True:
                comp_type, variable_hash, expr, value, target, next_addr = variable_comparisons[comp]
                if cases:
                    last_varhash = cases[-1].variable_hash
                else:
                    last_varhash = None
                if last_varhash is None or last_varhash == variable_hash:
                    if target == comp.addr:
                        # invalid
                        break
                    cases.append(Case(comp, comp_type, variable_hash, expr, value, target, next_addr))
                else:
                    # new variable!
                    if last_comp is not None:
                        cases.append(Case(last_comp, None, last_varhash, None, "default", comp.addr, None))
                    break

                if comp is not head:
                    # non-head node has at most one predecessor
                    if self._graph.in_degree[comp] > 1:
                        break

                successors = [succ for succ in self._graph.successors(comp) if succ is not comp]
                succ_addrs = {succ.addr for succ in successors}
                if target in succ_addrs:
                    next_comp_addr = next(iter(succ_addr for succ_addr in succ_addrs if succ_addr != target), None)
                    if next_comp_addr is None:
                        break
                    try:
                        next_comp = self._get_block(next_comp_addr)
                    except MultipleBlocksException:
                        # multiple blocks :/ it's possible that other optimization passes have duplicated the default
                        # node. check it.
                        next_comp_many = list(self._get_blocks(next_comp_addr))
                        if next_comp_many[0] not in variable_comparisons:
                            cases.append(Case(comp, None, variable_hash, expr, "default", next_comp_addr, None))
                        # otherwise we don't support it
                        break
                    assert next_comp is not None
                    if next_comp in variable_comparisons:
                        last_comp = comp
                        comp = next_comp
                        continue
                    cases.append(Case(comp, None, variable_hash, expr, "default", next_comp_addr, None))
                break

            if cases:
                v = cases[-1].variable_hash
                for idx, existing_cases in list(enumerate(varhash_to_caselists[v])):
                    if self.cases_issubset(existing_cases, cases):
                        varhash_to_caselists[v][idx] = cases
                        break
                    if self.cases_issubset(cases, existing_cases):
                        break
                else:
                    varhash_to_caselists[v].append(cases)

        for v, caselists in list(varhash_to_caselists.items()):
            for idx, cases in list(enumerate(caselists)):
                # filter: there should be at least two non-default cases
                if len([case for case in cases if case.value != "default"]) < 2:
                    caselists[idx] = None
                    continue

                # filter: no type-a node after the first case node
                if any(case for case in cases[1:] if case.value != "default" and case.node_type == "a"):
                    caselists[idx] = None
                    continue

                # filter: each case is only reachable from a case node
                all_case_nodes = {case.original_node for case in cases}
                skipped = False
                for case in cases:
                    target_nodes = [
                        succ for succ in self._graph.successors(case.original_node) if succ.addr == case.target
                    ]
                    if len(target_nodes) != 1:
                        caselists[idx] = None
                        skipped = True
                        break
                    target_node = target_nodes[0]
                    nonself_preds = {pred for pred in self._graph.predecessors(target_node) if pred.addr == case.target}
                    if not nonself_preds.issubset(all_case_nodes):
                        caselists[idx] = None
                        skipped = True
                        break

                if skipped:
                    continue

            varhash_to_caselists[v] = [cl for cl in caselists if cl is not None]

        return varhash_to_caselists

    @staticmethod
    def _find_switch_variable_comparison_type_a(
        node,
    ) -> Optional[Tuple[int, Expression, int, int, int]]:
        # the type a is the last statement is a var == constant comparison, but
        # there is more than one non-label statement in the block

        if isinstance(node, Block) and node.statements:
            stmt = node.statements[-1]
            if stmt is not None and stmt is not first_nonlabel_statement(node):
                if (
                    isinstance(stmt, ConditionalJump)
                    and isinstance(stmt.true_target, Const)
                    and isinstance(stmt.false_target, Const)
                ):
                    cond = stmt.condition
                    if isinstance(cond, BinaryOp):
                        if isinstance(cond.operands[1], Const):
                            variable_hash = StableVarExprHasher(cond.operands[0]).hash
                            value = cond.operands[1].value
                            if cond.op == "CmpEQ":
                                target = stmt.true_target.value
                                next_node_addr = stmt.false_target.value
                            elif cond.op == "CmpNE":
                                target = stmt.false_target.value
                                next_node_addr = stmt.true_target.value
                            else:
                                return None
                            return variable_hash, cond.operands[0], value, target, next_node_addr

        return None

    @staticmethod
    def _find_switch_variable_comparison_type_b(
        node,
    ) -> Optional[Tuple[int, Expression, int, int, int]]:
        # the type b is the last statement is a var == constant comparison, and
        # there is only one non-label statement

        if isinstance(node, Block):
            stmt = first_nonlabel_statement(node)
            if stmt is not None and stmt is node.statements[-1]:
                if (
                    isinstance(stmt, ConditionalJump)
                    and isinstance(stmt.true_target, Const)
                    and isinstance(stmt.false_target, Const)
                ):
                    cond = stmt.condition
                    if isinstance(cond, BinaryOp):
                        if isinstance(cond.operands[1], Const):
                            variable_hash = StableVarExprHasher(cond.operands[0]).hash
                            value = cond.operands[1].value
                            if cond.op == "CmpEQ":
                                target = stmt.true_target.value
                                next_node_addr = stmt.false_target.value
                            elif cond.op == "CmpNE":
                                target = stmt.false_target.value
                                next_node_addr = stmt.true_target.value
                            else:
                                return None
                            return variable_hash, cond.operands[0], value, target, next_node_addr

        return None

    @staticmethod
    def restore_graph(
        node, last_stmt: IncompleteSwitchCaseHeadStatement, graph: networkx.DiGraph, full_graph: networkx.DiGraph
    ):
        last_node = node
        ca_default = [
            (onode, value, target, a) for onode, value, target, a in last_stmt.case_addrs if value == "default"
        ]
        ca_others = [
            (onode, value, target, a) for onode, value, target, a in last_stmt.case_addrs if value != "default"
        ]

        # non-default nodes
        ca_others = {ca[0].addr: ca for ca in ca_others}
        # extract the AIL block from last_node
        last_block = last_node
        if isinstance(last_block, SequenceNode):
            last_block = last_block.nodes[-1]
        if isinstance(last_block, MultiNode):
            last_block = last_block.nodes[-1]
        assert isinstance(last_block, Block)
        next_node_addr = last_block.addr

        while next_node_addr is not None and next_node_addr in ca_others:
            onode, value, target, next_node_addr = ca_others[next_node_addr]
            onode: Block

            if first_nonlabel_statement(onode) is not onode.statements[-1]:
                onode = onode.copy(statements=[onode.statements[-1]])

            graph.add_edge(last_node, onode)
            full_graph.add_edge(last_node, onode)

            target_node = next(iter(nn for nn in full_graph if nn.addr == target))
            graph.add_edge(onode, target_node)
            full_graph.add_edge(onode, target_node)

            if graph.has_edge(node, target_node):
                graph.remove_edge(node, target_node)
            if full_graph.has_edge(node, target_node):
                full_graph.remove_edge(node, target_node)

            # update last_node
            last_node = onode

        # default nodes
        if ca_default:
            onode, value, target, _ = ca_default[0]
            default_target = next(iter(nn for nn in full_graph if nn.addr == target))
            graph.add_edge(last_node, default_target)
            full_graph.add_edge(last_node, default_target)

            if graph.has_edge(node, default_target):
                graph.remove_edge(node, default_target)
            if full_graph.has_edge(node, default_target):
                full_graph.remove_edge(node, default_target)

        # all good - remove the last statement in node
        remove_last_statement(node)

    @staticmethod
    def cases_issubset(cases_0: List[Case], cases_1: List[Case]) -> bool:
        """
        Test if cases_0 is a subset of cases_1.
        """

        if len(cases_0) > len(cases_1):
            return False
        for case in cases_0:
            if case not in cases_1:
                return False
        return True



import angr


def test_registration():
    s = angr.SimState(arch="AMD64")

    a1 = s.solver.BVS("a", 64, key=(1,), eternal=True)
    a2 = s.solver.BVS("a", 64, key=(1,), eternal=True)
    assert a1 is a2

    b1 = s.solver.BVS("b", 64, key=(2,), eternal=False)
    s1 = s.copy()
    s2 = s.copy()

    b2 = s1.solver.BVS("b", 64, key=(2,), eternal=False)
    b3 = s2.solver.BVS("b", 64, key=(2,), eternal=False)
    assert b1 is not b2
    assert b2 is not b3
    assert b1 is not b3

    a3 = s1.solver.BVS("a", 64, key=(1,), eternal=True)
    a4 = s2.solver.BVS("a", 64, key=(1,), eternal=True)
    assert a2 is a3
    assert a3 is a4

    assert len(list(s.solver.get_variables(1))) == 1
    assert len(list(s1.solver.get_variables(1))) == 1
    assert len(list(s2.solver.get_variables(1))) == 1

    assert len(list(s.solver.get_variables(2))) == 1
    assert len(list(s1.solver.get_variables(2))) == 2
    assert len(list(s2.solver.get_variables(2))) == 2

    assert list(s.solver.describe_variables(a1)) == [(1,)]
    assert list(s.solver.describe_variables(b1)) == [(2, 1)]
    assert sorted(list(s.solver.describe_variables(a1 + b1))) == [(1,), (2, 1)]



import os
import angr


def _bin(*s):
    return os.path.join(os.path.dirname(__file__), "..", "..", "binaries", "tests", *s)


def test_fauxware():
    project = angr.Project(_bin("i386", "fauxware"), auto_load_libs=False)

    result = [0, 0]

    @project.hook(0x80485DB)
    def check_backdoor(state):  # pylint:disable=unused-variable
        result[0] += 1
        if b"SOSNEAKY" in state.posix.dumps(0):
            result[1] = True
            project.terminate_execution()

    pg = project.execute()
    assert len(pg.deadended) != 3  # should terminate early
    assert result[1]


if __name__ == "__main__":
    test_fauxware()



import angr

######################################
# NoReturnUnconstrained
# Use in places you would put ReturnUnconstrained as a default action
# But the function shouldn't actually return
######################################


class NoReturnUnconstrained(angr.SimProcedure):  # pylint:disable=redefined-builtin
    NO_RET = True

    def run(self, **kwargs):  # pylint:disable=unused-argument
        self.exit(self.state.solver.Unconstrained("unconstrained_exit_code", self.state.arch.bits))



import logging
import unittest
import os

import angr

l = logging.getLogger("angr.tests")

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestBlockCache(unittest.TestCase):
    def test_block_cache(self):
        p = angr.Project(
            os.path.join(test_location, "x86_64", "fauxware"), translation_cache=True, auto_load_libs=False
        )
        b = p.factory.block(p.entry)
        assert p.factory.block(p.entry).vex is b.vex

        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), translation_cache=False)
        b = p.factory.block(p.entry)
        assert p.factory.block(p.entry).vex is not b.vex


if __name__ == "__main__":
    unittest.main()



import angr


def test_gettimeofday():
    proc = angr.SIM_PROCEDURES["posix"]["gettimeofday"]()

    s = angr.SimState(arch="amd64")
    s.regs.rdi = 0x8000
    s.regs.rsi = 0

    s.options.add(angr.options.USE_SYSTEM_TIMES)
    proc.execute(s)
    assert not s.mem[0x8000].qword.resolved.symbolic
    assert not s.mem[0x8008].qword.resolved.symbolic

    s.options.discard(angr.options.USE_SYSTEM_TIMES)
    proc.execute(s)
    assert s.mem[0x8000].qword.resolved.symbolic
    assert s.mem[0x8008].qword.resolved.symbolic


def test_clock_gettime():
    proc = angr.SIM_PROCEDURES["posix"]["clock_gettime"]()

    s = angr.SimState(arch="amd64")
    s.regs.rdi = 0
    s.regs.rsi = 0x8000

    s.options.add(angr.options.USE_SYSTEM_TIMES)
    proc.execute(s)
    assert not s.mem[0x8000].qword.resolved.symbolic
    assert not s.mem[0x8008].qword.resolved.symbolic

    s.options.discard(angr.options.USE_SYSTEM_TIMES)
    proc.execute(s)
    assert s.mem[0x8000].qword.resolved.symbolic
    assert s.mem[0x8008].qword.resolved.symbolic


if __name__ == "__main__":
    test_gettimeofday()
    test_clock_gettime()



import angr
import claripy

# all the input values were generated via
# [random.randrange(256) for _ in range(16)]
# then set into the input registers via gdb
# set $xmm0.v16_int8 = {...}
# then read out as uint128s
# p/x $xmm0.uint128
# then single stepped and the result read out


def test_irop_perm():
    p = angr.load_shellcode("vpshufb xmm0,xmm1,xmm2", "amd64")

    # concrete test
    s1 = p.factory.blank_state()
    s1.regs.xmm1 = 0x3C899A56814EE9B84C7B5D8394C85881
    s1.regs.xmm2 = 0xA55C66A2CDEF1CBCD72B42078D1B7F8B
    s2 = s1.step(num_inst=1).successors[0]
    assert (s2.regs.xmm0 == 0x00567B00000056000081C84C00813C00).is_true()

    # symbolic test
    s3 = p.factory.blank_state()
    s3.regs.xmm1 = claripy.BVS("xmm1", 128)
    s3.regs.xmm2 = claripy.BVS("xmm2", 128)
    s4 = s3.step(num_inst=1).successors[0]
    s4.solver.add(s4.regs.xmm2 == 0xA55C66A2CDEF1CBCD72B42078D1B7F8B)
    s4.solver.add(s4.regs.xmm0 == 0x00567B00000056000081C84C00813C00)
    assert s4.solver.solution(s4.regs.xmm1, 0x3C899A56814EE9B84C7B5D8394C85881)


def test_irop_mulhi():
    p = angr.load_shellcode("vpmulhw xmm0,xmm1,xmm2", "amd64")

    # concrete test
    s1 = p.factory.blank_state()
    s1.regs.xmm1 = 0x3ACA92553C2526D4F20987AEAB250255
    s1.regs.xmm2 = 0x1AEBCB281463274EC3CE6473619A8541
    s2 = s1.step(num_inst=1).successors[0]
    assert (s2.regs.xmm0 == 0x62E16A304CA05F60348D0C9DFA5FEE1).is_true()


def test_irop_catevenlanes():
    p = angr.load_shellcode("pmulhrsw xmm0, xmm1", "amd64")

    # concrete test
    s1 = p.factory.blank_state()
    s1.regs.xmm0 = 0x4713E06BF3235E97CA8CFDE0647D65FD
    s1.regs.xmm1 = 0x31F1F86DA1DCE7DE252ADC78160E1016
    s2 = s1.step(num_inst=1).successors[0]
    assert (s2.regs.xmm0 == 0x1BBB01DE0976EE2BF07B009711500CD1).is_true()


def test_saturating_packing():
    # SaturateSignedWordToUnsignedByte
    p = angr.load_shellcode("vpackuswb xmm1, xmm0, xmm0", arch="amd64")
    s = p.factory.blank_state()
    s.regs.xmm0 = 0x0000_0001_7FFE_7FFF_8000_8001_FFFE_FFFF
    s = s.step(num_inst=1).successors[0]
    assert (s.regs.xmm1 == 0x00_01_FF_FF_00_00_00_00_0001FFFF00000000).is_true()

    # "Pack with unsigned saturation"
    p = angr.load_shellcode("vpackusdw xmm1, xmm0, xmm0", arch="amd64")
    s = p.factory.blank_state()
    s.regs.xmm0 = 0x00000001_7FFFFFFE_80000001_FFFFFFFE
    s = s.step(num_inst=1).successors[0]
    assert (s.regs.xmm1 == 0x0001_FFFF_0000_0000_0001FFFF00000000).is_true()

    # SaturateSignedWordToSignedByte
    p = angr.load_shellcode("vpacksswb xmm1, xmm0, xmm0", arch="amd64")
    s = p.factory.blank_state()
    s.regs.xmm0 = 0x0000_0001_7FFE_7FFF_8000_8001_FFFE_FFFF
    s = s.step(num_inst=1).successors[0]
    assert (s.regs.xmm1 == 0x00_01_7F_7F_80_80_FE_FF_00017F7F8080FEFF).is_true()


if __name__ == "__main__":
    test_irop_perm()
    test_irop_mulhi()
    test_irop_catevenlanes()
    test_saturating_packing()



import angr

import logging

l = logging.getLogger("angr_tests.dataflowgraph")

import os

test_location = os.path.dirname(os.path.realpath(__file__))


def test_sprintf():
    p = angr.Project(
        os.path.join(test_location, "..", "..", "binaries", "tests", "x86_64", "sprintf_test"), auto_load_libs=False
    )
    a = p.factory.simulation_manager().explore(find=0x4005C0)
    state = a.found[0]

    str1 = state.solver.eval(state.memory.load(0x600AD0, 13), cast_to=bytes)
    assert str1 == b"Immediate: 3\n"

    str2 = state.solver.eval(state.memory.load(0x600A70, 7), cast_to=bytes)
    assert str2 == b"Int: 3\n"

    str3 = state.solver.eval(state.memory.load(0x600AB0, 8), cast_to=bytes)
    assert str3 == b"Char: c\n"

    str4 = state.solver.eval(state.memory.load(0x600A50, 14), cast_to=bytes)
    assert str4 == b"Uninit int: 0\n"

    str5 = state.solver.eval(state.memory.load(0x600A90, 24), cast_to=bytes)
    assert str5 == b"Str on stack: A string.\n"

    str6 = state.solver.eval(state.memory.load(0x600A30, 21), cast_to=bytes)
    assert str6 == b"Global str: GLOB_STR\n"


if __name__ == "__main__":
    test_sprintf()



import os

import angr

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")
bina = os.path.join(test_location, "x86_64", "test_project_resolve_simproc")

# We voluntarily don't use SimProcedures for 'rand' and 'sleep' because we want
# to step into their lib code.


def test_bina():
    # auto_load_libs can't be disabled as the testcase fails
    p = angr.Project(bina, exclude_sim_procedures_list=["rand", "sleep"], load_options={"auto_load_libs": True})

    # Make sure external functions are not replaced with a SimProcedure
    sleep_jmpslot = p.loader.main_object.jmprel["sleep"]
    rand_jmpslot = p.loader.main_object.jmprel["rand"]
    read_jmpslot = p.loader.main_object.jmprel["read"]

    sleep_addr = p.loader.memory.unpack_word(sleep_jmpslot.rebased_addr)
    rand_addr = p.loader.memory.unpack_word(rand_jmpslot.rebased_addr)
    read_addr = p.loader.memory.unpack_word(read_jmpslot.rebased_addr)

    libc_sleep_addr = p.loader.shared_objects["libc.so.6"].get_symbol("sleep").rebased_addr
    libc_rand_addr = p.loader.shared_objects["libc.so.6"].get_symbol("rand").rebased_addr

    assert sleep_addr == libc_sleep_addr
    assert rand_addr == libc_rand_addr
    assert p.is_hooked(read_addr)
    assert "read" in str(p._sim_procedures[read_addr])


if __name__ == "__main__":
    test_bina()



import angr

import logging

l = logging.getLogger("angr.tests")

import os

test_location = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "..", "binaries", "tests")

arches = {"mips", "mipsel", "mips64", "x86_64", "ppc", "ppc64"}


def run_test_syscalls(arch):
    # import ipdb; ipdb.set_trace()
    p = angr.Project(os.path.join(test_location, arch, "test_ioctl"), exclude_sim_procedures_list=["ioctl"])
    p.simos.syscall_library.procedures.pop("ioctl", None)

    s = p.factory.entry_state()

    simgr = p.factory.simulation_manager(thing=s)
    simgr.run()
    assert len(simgr.deadended) == 2, "for these architectures, libc checks if the bit is set. make sure it branches"


if __name__ == "__main__":
    for arch in arches:
        run_test_syscalls(arch)
